@@ -61,6 +61,7 @@ usage() {
     echo "        '-v' display the script version"
     echo "        '-w' working directory for image building (default: .)"
     echo "        '-x' enable tracing for diskimage-builder"
+    echo "        '-y' enable FIPS 140-2 mode in the amphora image"
     echo
     exit 1
 }
@@ -91,7 +92,7 @@ dib_enable_tracing=
 
 AMP_LOGFILE=""
 
-while getopts "a:b:c:d:efg:hi:k:l:no:pt:r:s:vw:x" opt; do
+while getopts "a:b:c:d:efg:hi:k:l:no:pt:r:s:vw:xy" opt; do
     case $opt in
         a)
             AMP_ARCH=$OPTARG
@@ -207,6 +208,8 @@ while getopts "a:b:c:d:efg:hi:k:l:no:pt:r:s:vw:x" opt; do
         ;;
         x)  dib_enable_tracing=1
         ;;
+        y)  AMP_ENABLE_FIPS=1
+        ;;
         *)
             usage
         ;;
@@ -256,6 +259,8 @@ AMP_ENABLE_FULL_MAC_SECURITY=${AMP_ENABLE_FULL_MAC_SECURITY:-0}
 
 AMP_DISABLE_TMP_FS=${AMP_DISABLE_TMP_FS:-""}
 
+AMP_ENABLE_FIPS=${AMP_ENABLE_FIPS:-0}
+
 if [[ "$AMP_BASEOS" =~ ^(rhel|fedora)$ ]] && [[ "$AMP_IMAGESIZE" -lt 3 ]]; then
     echo "RHEL/Fedora based amphora requires an image size of at least 3GB"
     exit 1
@@ -440,10 +445,13 @@ AMP_element_sequence="$AMP_element_sequence sos"
 AMP_element_sequence="$AMP_element_sequence cloud-init-datasources"
 AMP_element_sequence="$AMP_element_sequence remove-default-ints"
 
-if [ "$AMP_ENABLE_FULL_MAC_SECURITY" -ne 1 ]; then
-    # SELinux systems
-    if [ "${AMP_BASEOS}" = "centos-minimal" ] || [ "${AMP_BASEOS}" = "fedora" ] || [ "${AMP_BASEOS}" = "rhel" ]; then
+# SELinux systems
+if [ "${AMP_BASEOS}" = "centos-minimal" ] || [ "${AMP_BASEOS}" = "fedora" ] || [ "${AMP_BASEOS}" = "rhel" ]; then
+    if [ "$AMP_ENABLE_FULL_MAC_SECURITY" -ne 1 ]; then
         AMP_element_sequence="$AMP_element_sequence selinux-permissive"
+    else
+        # If SELinux is enforced, the amphora image requires the amphora-selinux policies
+        AMP_element_sequence="$AMP_element_sequence amphora-selinux"
     fi
 fi
 
@@ -471,6 +479,11 @@ if [ "$AMP_DISABLE_SSHD" -eq 1 ]; then
     AMP_element_sequence="$AMP_element_sequence remove-sshd"
 fi
 
+# Enable FIPS if requested
+if [ "$AMP_ENABLE_FIPS" -eq 1 ]; then
+    AMP_element_sequence="$AMP_element_sequence amphora-fips"
+fi
+
 # Allow full elements override
 if [ "$DIB_ELEMENTS" ]; then
     AMP_element_sequence="$DIB_ELEMENTS"
