@@ -15,11 +15,14 @@
 import csv
 import socket
 
-import six
+from oslo_log import log as logging
 
 from octavia.common import constants as consts
+from octavia.common import utils as octavia_utils
 from octavia.i18n import _
 
+LOG = logging.getLogger(__name__)
+
 
 class HAProxyQuery(object):
     """Class used for querying the HAProxy statistics socket.
@@ -30,9 +33,9 @@ class HAProxyQuery(object):
     """
 
     def __init__(self, stats_socket):
-        """stats_socket
+        """Initialize the class
 
-            Path to the HAProxy statistics socket file.
+        :param stats_socket: Path to the HAProxy statistics socket file.
         """
 
         self.socket = stats_socket
@@ -48,18 +51,19 @@ class HAProxyQuery(object):
 
         try:
             sock.connect(self.socket)
-        except socket.error:
-            raise Exception(_("HAProxy '{0}' query failed.").format(query))
+        except socket.error as e:
+            raise Exception(
+                _("HAProxy '{0}' query failed.").format(query)) from e
 
         try:
-            sock.send(six.b(query + '\n'))
-            data = u''
+            sock.send(octavia_utils.b(query + '\n'))
+            data = ''
             while True:
                 x = sock.recv(1024)
                 if not x:
                     break
                 data += x.decode('ascii') if (
-                    isinstance(x, six.binary_type)) else x
+                    isinstance(x, bytes)) else x
             return data.rstrip()
         finally:
             sock.close()
@@ -68,7 +72,7 @@ class HAProxyQuery(object):
         """Get and parse output from 'show info' command."""
         results = self._query('show info')
 
-        dict_results = dict()
+        dict_results = {}
         for r in results.split('\n'):
             vals = r.split(":", 1)
             dict_results[vals[0].strip()] = vals[1].strip()
@@ -115,20 +119,42 @@ class HAProxyQuery(object):
         for line in results:
             # pxname: pool, svname: server_name, status: status
 
-            # Due to a bug in some versions of HAProxy, DRAIN mode isn't
-            # calculated correctly, but we can spoof the correct value here.
-            if line['status'] == consts.UP and line['weight'] == 0:
-                line['status'] = consts.DRAIN
-
             if line['pxname'] not in final_results:
                 final_results[line['pxname']] = dict(members={})
 
             if line['svname'] == 'BACKEND':
+                # BACKEND describes a pool of servers in HAProxy
                 pool_id, listener_id = line['pxname'].split(':')
                 final_results[line['pxname']]['pool_uuid'] = pool_id
                 final_results[line['pxname']]['listener_uuid'] = listener_id
                 final_results[line['pxname']]['status'] = line['status']
             else:
+                # Due to a bug in some versions of HAProxy, DRAIN mode isn't
+                # calculated correctly, but we can spoof the correct
+                # value here.
+                if line['status'] == consts.UP and line['weight'] == '0':
+                    line['status'] = consts.DRAIN
+
                 final_results[line['pxname']]['members'][line['svname']] = (
                     line['status'])
         return final_results
+
+    def save_state(self, state_file_path):
+        """Save haproxy connection state to a file.
+
+        :param state_file_path: Absolute path to the state file
+
+        :returns: bool (True if success, False otherwise)
+        """
+
+        try:
+            result = self._query('show servers state')
+            # No need for binary mode, the _query converts bytes to ascii.
+            with open(state_file_path, 'w', encoding='utf-8') as fh:
+                fh.write(result)
+            return True
+        except Exception as e:
+            # Catch any exception - may be socket issue, or write permission
+            # issue as well.
+            LOG.warning("Unable to save state: %r", e)
+            return False
