@@ -162,6 +162,8 @@ class AmphoraLoadBalancerDriver(object, metaclass=abc.ABCMeta):
         :param amphorae_network_config: A data model containing information
                                         about the subnets and ports that an
                                         amphorae owns.
+        :type amphorae_network_config: octavia.network.data_models.
+                                       AmphoraNetworkConfig
         :param vrrp_port: VRRP port associated with the load balancer
         :type vrrp_port: octavia.network.data_models.Port
 
@@ -175,13 +177,18 @@ class AmphoraLoadBalancerDriver(object, metaclass=abc.ABCMeta):
         the vip, such as bring up interfaces.
         """
 
-    def post_network_plug(self, amphora, port):
+    def post_network_plug(self, amphora, port, amphora_network_config):
         """Called after amphora added to network
 
         :param amphora: amphora object, needs id and network ip(s)
         :type amphora: octavia.db.models.Amphora
         :param port: contains information of the plugged port
         :type port: octavia.network.data_models.Port
+        :param amphora_network_config: A data model containing information
+                                        about the subnets and ports that an
+                                        amphorae owns.
+        :type amphora_network_config: octavia.network.data_models.
+                                      AmphoraNetworkConfig
 
         This method is optional to implement.  After adding an amphora to a
         network, there may be steps necessary on the amphora to allow it to
@@ -189,23 +196,6 @@ class AmphoraLoadBalancerDriver(object, metaclass=abc.ABCMeta):
         neutron network to utilize.
         """
 
-    def start_health_check(self, health_mixin):
-        """Start health checks.
-
-        :param health_mixin: health mixin object
-        :type health_mixin: HealthMixin
-
-        Starts listener process and calls HealthMixin to update
-        databases information.
-        """
-
-    def stop_health_check(self):
-        """Stop health checks.
-
-        Stops listener process and calls HealthMixin to update
-        databases information.
-        """
-
     def upload_cert_amp(self, amphora, pem_file):
         """Upload cert info to the amphora.
 
@@ -242,47 +232,6 @@ class AmphoraLoadBalancerDriver(object, metaclass=abc.ABCMeta):
         """
 
 
-class HealthMixin(object, metaclass=abc.ABCMeta):
-    @abc.abstractmethod
-    def update_health(self, health):
-        """Return ceilometer ready health
-
-        :param health: health information emitted from the amphora
-        :type health: bool
-        :returns: return health
-
-        At this moment, we just build the basic structure for testing, will
-        add more function along with the development, eventually, we want it
-        return:
-        map: {"amphora-status":HEALTHY, loadbalancers: {"loadbalancer-id":
-        {"loadbalancer-status": HEALTHY,
-        "listeners":{"listener-id":{"listener-status":HEALTHY,
-        "nodes":{"node-id":HEALTHY, ...}}, ...}, ...}}
-        only items whose health has changed need to be submitted
-        awesome update code
-        """
-
-
-class StatsMixin(object, metaclass=abc.ABCMeta):
-    @abc.abstractmethod
-    def update_stats(self, stats):
-        """Return ceilometer ready stats
-
-        :param stats: statistic information emitted from the amphora
-        :type stats: string
-        :returns: return stats
-
-        At this moment, we just build the basic structure for testing, will
-        add more function along with the development, eventually, we want it
-        return:
-        uses map {"loadbalancer-id":{"listener-id":
-        {"bytes-in": 123, "bytes_out":123, "active_connections":123,
-        "total_connections", 123}, ...}
-        elements are named to keep it extsnsible for future versions
-        awesome update code and code to send to ceilometer
-        """
-
-
 class VRRPDriverMixin(object, metaclass=abc.ABCMeta):
     """Abstract mixin class for VRRP support in loadbalancer amphorae
 
