@@ -21,6 +21,7 @@ import warnings
 
 from oslo_context import context as oslo_context
 from oslo_log import log as logging
+from oslo_utils.secretutils import md5
 import requests
 import simplejson
 from stevedore import driver as stevedore_driver
@@ -38,6 +39,7 @@ from octavia.common.tls_utils import cert_parser
 from octavia.common import utils
 from octavia.db import api as db_apis
 from octavia.db import repositories as repo
+from octavia.network import data_models as network_models
 
 
 LOG = logging.getLogger(__name__)
@@ -77,15 +79,17 @@ class HaproxyAmphoraLoadBalancerDriver(
             connection_logging=CONF.haproxy_amphora.connection_logging)
         self.lvs_jinja = jinja_udp_cfg.LvsJinjaTemplater()
 
-    def _get_haproxy_versions(self, amphora):
+    def _get_haproxy_versions(self, amphora, timeout_dict=None):
         """Get major and minor version number from haproxy
 
         Example: ['1', '6']
 
         :returns version_list: A list with the major and minor numbers
         """
-        self._populate_amphora_api_version(amphora)
-        amp_info = self.clients[amphora.api_version].get_info(amphora)
+        self._populate_amphora_api_version(
+            amphora, timeout_dict=timeout_dict)
+        amp_info = self.clients[amphora.api_version].get_info(
+            amphora, timeout_dict=timeout_dict)
         haproxy_version_string = amp_info['haproxy_version']
 
         return haproxy_version_string.split('.')[:2]
@@ -135,7 +139,8 @@ class HaproxyAmphoraLoadBalancerDriver(
             return
 
         # Check which HAProxy version is on the amp
-        haproxy_versions = self._get_haproxy_versions(amphora)
+        haproxy_versions = self._get_haproxy_versions(
+            amphora, timeout_dict=timeout_dict)
         # Check which config style to use
         api_version = self._populate_amphora_api_version(amphora)
         if api_version[0] == 0 and api_version[1] <= 5:  # 0.5 or earlier
@@ -258,15 +263,18 @@ class HaproxyAmphoraLoadBalancerDriver(
         self._populate_amphora_api_version(amp)
         self.clients[amp.api_version].update_cert_for_rotation(amp, pem)
 
-    def _apply(self, func_name, loadbalancer, amphora=None, *args):
+    def _apply(self, func_name, loadbalancer, amphora=None, *args, **kwargs):
         if amphora is None:
             amphorae = loadbalancer.amphorae
         else:
             amphorae = [amphora]
 
+        timeout_dict = args[0]
+
         for amp in amphorae:
             if amp.status != consts.DELETED:
-                api_version = self._populate_amphora_api_version(amp)
+                api_version = self._populate_amphora_api_version(
+                    amp, timeout_dict=timeout_dict)
                 # Check which config style to use
                 if api_version[0] == 0 and api_version[1] <= 5:
                     # 0.5 or earlier
@@ -368,11 +376,14 @@ class HaproxyAmphoraLoadBalancerDriver(
             self.clients[amphora.api_version].delete_listener(
                 amphora, listener.load_balancer.id)
 
-    def get_info(self, amphora, raise_retry_exception=False):
+    def get_info(self, amphora, raise_retry_exception=False,
+                 timeout_dict=None):
         self._populate_amphora_api_version(
-            amphora, raise_retry_exception=raise_retry_exception)
+            amphora, raise_retry_exception=raise_retry_exception,
+            timeout_dict=timeout_dict)
         return self.clients[amphora.api_version].get_info(
-            amphora, raise_retry_exception=raise_retry_exception)
+            amphora, raise_retry_exception=raise_retry_exception,
+            timeout_dict=timeout_dict)
 
     def get_diagnostics(self, amphora):
         pass
@@ -380,23 +391,33 @@ class HaproxyAmphoraLoadBalancerDriver(
     def finalize_amphora(self, amphora):
         pass
 
+    def _build_net_info(self, port, amphora, subnet, mtu=None):
+        # NOTE(blogan): using the vrrp port here because that
+        # is what the allowed address pairs network driver sets
+        # this particular port to.  This does expose a bit of
+        # tight coupling between the network driver and amphora
+        # driver.  We will need to revisit this to try and remove
+        # this tight coupling.
+        # NOTE (johnsom): I am loading the vrrp_ip into the
+        # net_info structure here so that I don't break
+        # compatibility with old amphora agent versions.
+        host_routes = [{'nexthop': hr[consts.NEXTHOP],
+                        'destination': hr[consts.DESTINATION]}
+                       for hr in subnet[consts.HOST_ROUTES]]
+        net_info = {'subnet_cidr': subnet[consts.CIDR],
+                    'gateway': subnet[consts.GATEWAY_IP],
+                    'mac_address': port[consts.MAC_ADDRESS],
+                    'vrrp_ip': amphora[consts.VRRP_IP],
+                    'mtu': mtu or port[consts.NETWORK][consts.MTU],
+                    'host_routes': host_routes}
+        return net_info
+
     def post_vip_plug(self, amphora, load_balancer, amphorae_network_config,
                       vrrp_port=None, vip_subnet=None):
         if amphora.status != consts.DELETED:
             self._populate_amphora_api_version(amphora)
             if vip_subnet is None:
-                subnet = amphorae_network_config.get(amphora.id).vip_subnet
-            else:
-                subnet = vip_subnet
-            # NOTE(blogan): using the vrrp port here because that
-            # is what the allowed address pairs network driver sets
-            # this particular port to.  This does expose a bit of
-            # tight coupling between the network driver and amphora
-            # driver.  We will need to revisit this to try and remove
-            # this tight coupling.
-            # NOTE (johnsom): I am loading the vrrp_ip into the
-            # net_info structure here so that I don't break
-            # compatibility with old amphora agent versions.
+                vip_subnet = amphorae_network_config.get(amphora.id).vip_subnet
             if vrrp_port is None:
                 port = amphorae_network_config.get(amphora.id).vrrp_port
                 mtu = port.network.mtu
@@ -405,15 +426,9 @@ class HaproxyAmphoraLoadBalancerDriver(
                 mtu = port.network['mtu']
             LOG.debug("Post-VIP-Plugging with vrrp_ip %s vrrp_port %s",
                       amphora.vrrp_ip, port.id)
-            host_routes = [{'nexthop': hr.nexthop,
-                            'destination': hr.destination}
-                           for hr in subnet.host_routes]
-            net_info = {'subnet_cidr': subnet.cidr,
-                        'gateway': subnet.gateway_ip,
-                        'mac_address': port.mac_address,
-                        'vrrp_ip': amphora.vrrp_ip,
-                        'mtu': mtu,
-                        'host_routes': host_routes}
+            net_info = self._build_net_info(
+                port.to_dict(recurse=True), amphora.to_dict(),
+                vip_subnet.to_dict(recurse=True), mtu)
             try:
                 self.clients[amphora.api_version].plug_vip(
                     amphora, load_balancer.vip.ip_address, net_info)
@@ -422,7 +437,7 @@ class HaproxyAmphoraLoadBalancerDriver(
                             'skipping post_vip_plug',
                             {'mac': port.mac_address})
 
-    def post_network_plug(self, amphora, port):
+    def post_network_plug(self, amphora, port, amphora_network_config):
         fixed_ips = []
         for fixed_ip in port.fixed_ips:
             host_routes = [{'nexthop': hr.nexthop,
@@ -430,11 +445,25 @@ class HaproxyAmphoraLoadBalancerDriver(
                            for hr in fixed_ip.subnet.host_routes]
             ip = {'ip_address': fixed_ip.ip_address,
                   'subnet_cidr': fixed_ip.subnet.cidr,
-                  'host_routes': host_routes}
+                  'host_routes': host_routes,
+                  'gateway': fixed_ip.subnet.gateway_ip}
             fixed_ips.append(ip)
         port_info = {'mac_address': port.mac_address,
                      'fixed_ips': fixed_ips,
                      'mtu': port.network.mtu}
+        if port.id == amphora.vrrp_port_id:
+            if isinstance(amphora_network_config,
+                          network_models.AmphoraNetworkConfig):
+                amphora_network_config = amphora_network_config.to_dict(
+                    recurse=True)
+            # We have to special-case sharing the vrrp port and pass through
+            # enough extra information to populate the whole VIP port
+            net_info = self._build_net_info(
+                port.to_dict(recurse=True), amphora.to_dict(),
+                amphora_network_config[consts.VIP_SUBNET],
+                port.network.mtu)
+            net_info['vip'] = amphora.ha_ip
+            port_info['vip_net_info'] = net_info
         try:
             self._populate_amphora_api_version(amphora)
             self.clients[amphora.api_version].plug_network(amphora, port_info)
@@ -468,20 +497,21 @@ class HaproxyAmphoraLoadBalancerDriver(
         if amphora and obj_id:
             for cert in certs:
                 pem = cert_parser.build_pem(cert)
-                md5 = hashlib.md5(pem).hexdigest()  # nosec
+                md5sum = md5(pem, usedforsecurity=False).hexdigest()  # nosec
                 name = '{id}.pem'.format(id=cert.id)
                 cert_filename_list.append(
                     os.path.join(
                         CONF.haproxy_amphora.base_cert_dir, obj_id, name))
-                self._upload_cert(amphora, obj_id, pem, md5, name)
+                self._upload_cert(amphora, obj_id, pem, md5sum, name)
 
             if certs:
                 # Build and upload the crt-list file for haproxy
                 crt_list = "\n".join(cert_filename_list)
                 crt_list = f'{crt_list}\n'.encode('utf-8')
-                md5 = hashlib.md5(crt_list).hexdigest()  # nosec
+                md5sum = md5(crt_list,
+                             usedforsecurity=False).hexdigest()  # nosec
                 name = '{id}.pem'.format(id=listener.id)
-                self._upload_cert(amphora, obj_id, crt_list, md5, name)
+                self._upload_cert(amphora, obj_id, crt_list, md5sum, name)
         return {'tls_cert': tls_cert, 'sni_certs': sni_certs}
 
     def _process_secret(self, listener, secret_ref, amphora=None, obj_id=None):
@@ -497,13 +527,13 @@ class HaproxyAmphoraLoadBalancerDriver(
             secret = secret.encode('utf-8')
         except AttributeError:
             pass
-        md5 = hashlib.md5(secret).hexdigest()  # nosec
+        md5sum = md5(secret, usedforsecurity=False).hexdigest()  # nosec
         id = hashlib.sha1(secret).hexdigest()  # nosec
         name = '{id}.pem'.format(id=id)
 
         if amphora and obj_id:
             self._upload_cert(
-                amphora, obj_id, pem=secret, md5=md5, name=name)
+                amphora, obj_id, pem=secret, md5sum=md5sum, name=name)
         return name
 
     def _process_listener_pool_certs(self, listener, amphora, obj_id):
@@ -511,7 +541,7 @@ class HaproxyAmphoraLoadBalancerDriver(
         #         'client_cert': client_full_filename,
         #         'ca_cert': ca_cert_full_filename,
         #         'crl': crl_full_filename}}
-        pool_certs_dict = dict()
+        pool_certs_dict = {}
         for pool in listener.pools:
             if pool.id not in pool_certs_dict:
                 pool_certs_dict[pool.id] = self._process_pool_certs(
@@ -525,7 +555,7 @@ class HaproxyAmphoraLoadBalancerDriver(
         return pool_certs_dict
 
     def _process_pool_certs(self, listener, pool, amphora, obj_id):
-        pool_cert_dict = dict()
+        pool_cert_dict = {}
 
         # Handle the client cert(s) and key
         if pool.tls_certificate_id:
@@ -536,10 +566,11 @@ class HaproxyAmphoraLoadBalancerDriver(
                 pem = pem.encode('utf-8')
             except AttributeError:
                 pass
-            md5 = hashlib.md5(pem).hexdigest()  # nosec
+            md5sum = md5(pem, usedforsecurity=False).hexdigest()  # nosec
             name = '{id}.pem'.format(id=tls_cert.id)
             if amphora and obj_id:
-                self._upload_cert(amphora, obj_id, pem=pem, md5=md5, name=name)
+                self._upload_cert(amphora, obj_id, pem=pem,
+                                  md5sum=md5sum, name=name)
             pool_cert_dict['client_cert'] = os.path.join(
                 CONF.haproxy_amphora.base_cert_dir, obj_id, name)
         if pool.ca_tls_certificate_id:
@@ -555,10 +586,10 @@ class HaproxyAmphoraLoadBalancerDriver(
 
         return pool_cert_dict
 
-    def _upload_cert(self, amp, listener_id, pem, md5, name):
+    def _upload_cert(self, amp, listener_id, pem, md5sum, name):
         try:
             if self.clients[amp.api_version].get_cert_md5sum(
-                    amp, listener_id, name, ignore=(404,)) == md5:
+                    amp, listener_id, name, ignore=(404,)) == md5sum:
                 return
         except exc.NotFound:
             pass
@@ -808,8 +839,10 @@ class AmphoraAPIClient0_5(AmphoraAPIClientBase):
             amp, 'listeners/{listener_id}'.format(listener_id=listener_id))
         return exc.check_exception(r, (404,))
 
-    def get_info(self, amp, raise_retry_exception=False):
-        r = self.get(amp, "info", raise_retry_exception=raise_retry_exception)
+    def get_info(self, amp, raise_retry_exception=False,
+                 timeout_dict=None):
+        r = self.get(amp, "info", raise_retry_exception=raise_retry_exception,
+                     timeout_dict=timeout_dict)
         if exc.check_exception(r):
             return r.json()
         return None
@@ -937,8 +970,10 @@ class AmphoraAPIClient1_0(AmphoraAPIClientBase):
             amp, 'listeners/{object_id}'.format(object_id=object_id))
         return exc.check_exception(r, (404,))
 
-    def get_info(self, amp, raise_retry_exception=False):
-        r = self.get(amp, "info", raise_retry_exception=raise_retry_exception)
+    def get_info(self, amp, raise_retry_exception=False,
+                 timeout_dict=None):
+        r = self.get(amp, "info", raise_retry_exception=raise_retry_exception,
+                     timeout_dict=timeout_dict)
         if exc.check_exception(r):
             return r.json()
         return None
