@@ -29,7 +29,7 @@ from octavia.api.v2.types import member as member_types
 from octavia.common import constants
 from octavia.common import data_models
 from octavia.common import exceptions
-import octavia.common.validate as validate
+from octavia.common import validate
 from octavia.db import api as db_api
 from octavia.db import prepare as db_prepare
 
@@ -119,16 +119,10 @@ class MemberController(base.BaseController):
         """Validate creating member on pool."""
         try:
             return self.repositories.member.create(lock_session, **member_dict)
-        except odb_exceptions.DBDuplicateEntry as de:
-            column_list = ['pool_id', 'ip_address', 'protocol_port']
-            constraint_list = ['uq_member_pool_id_address_protocol_port']
-            if ['id'] == de.columns:
-                raise exceptions.IDAlreadyExists()
-            if (set(column_list) == set(de.columns) or
-                    set(constraint_list) == set(de.columns)):
-                raise exceptions.DuplicateMemberEntry(
-                    ip_address=member_dict.get('ip_address'),
-                    port=member_dict.get('protocol_port'))
+        except odb_exceptions.DBDuplicateEntry as e:
+            raise exceptions.DuplicateMemberEntry(
+                ip_address=member_dict.get('ip_address'),
+                port=member_dict.get('protocol_port')) from e
         except odb_exceptions.DBError as e:
             # TODO(blogan): will have to do separate validation protocol
             # before creation or update since the exception messages
@@ -343,13 +337,6 @@ class MembersController(MemberController):
             self._auth_validate_action(context, project_id,
                                        constants.RBAC_DELETE)
 
-        # Validate member subnets
-        for member in members:
-            if member.subnet_id and not validate.subnet_exists(
-                    member.subnet_id, context=context):
-                raise exceptions.NotFound(resource='Subnet',
-                                          id=member.subnet_id)
-
         # Load the driver early as it also provides validation
         driver = driver_factory.get_driver(provider)
 
@@ -395,8 +382,27 @@ class MembersController(MemberController):
                     resource=data_models.Member._name())
 
             provider_members = []
+            valid_subnets = set()
             # Create new members
             for m in new_members:
+                # NOTE(mnaser): In order to avoid hitting the Neutron API hard
+                # when creating many new members, we cache the
+                # validation results. We also validate new
+                # members only since subnet ID is immutable.
+                # If the member doesn't have a subnet, or the subnet is
+                # already valid, move on. Run validate and add it to
+                # cache otherwise.
+                if m.subnet_id and m.subnet_id not in valid_subnets:
+                    # If the subnet does not exist,
+                    # raise an exception and get out.
+                    if not validate.subnet_exists(
+                            m.subnet_id, context=context):
+                        raise exceptions.NotFound(
+                            resource='Subnet', id=m.subnet_id)
+
+                    # Mark the subnet as valid for future runs.
+                    valid_subnets.add(m.subnet_id)
+
                 m = m.to_dict(render_unsets=False)
                 m['project_id'] = db_pool.project_id
                 created_member = self._graph_create(lock_session, m)
