@@ -298,7 +298,7 @@ SUPPORTED_TASKFLOW_ENGINE_TYPES = ['serial', 'parallel']
 # Task/Flow constants
 ACTIVE_CONNECTIONS = 'active_connections'
 ADD_NICS = 'add_nics'
-ADDED_PORTS = 'added_ports'
+ADD_SUBNETS = 'add_subnets'
 ADMIN_STATE_UP = 'admin_state_up'
 ALLOWED_ADDRESS_PAIRS = 'allowed_address_pairs'
 AMP_DATA = 'amp_data'
@@ -330,9 +330,11 @@ CREATED_AT = 'created_at'
 CRL_CONTAINER_ID = 'crl_container_id'
 DEFAULT_TLS_CONTAINER_DATA = 'default_tls_container_data'
 DELETE_NICS = 'delete_nics'
+DELETE_SUBNETS = 'delete_subnets'
 DELTA = 'delta'
 DELTAS = 'deltas'
 DESCRIPTION = 'description'
+DESTINATION = 'destination'
 DEVICE_OWNER = 'device_owner'
 ENABLED = 'enabled'
 FAILED_AMP_VRRP_PORT_ID = 'failed_amp_vrrp_port_id'
@@ -342,6 +344,7 @@ FAILOVER_AMPHORA_ID = 'failover_amphora_id'
 FIELDS = 'fields'
 FIXED_IPS = 'fixed_ips'
 FLAVOR_ID = 'flavor_id'
+GATEWAY_IP = 'gateway_ip'
 HA_IP = 'ha_ip'
 HA_PORT_ID = 'ha_port_id'
 HEALTH_MON = 'health_mon'
@@ -349,6 +352,7 @@ HEALTH_MONITOR = 'health_monitor'
 HEALTH_MONITOR_ID = 'health_monitor_id'
 HEALTHMONITOR_ID = 'healthmonitor_id'
 HEALTH_MONITOR_UPDATES = 'health_monitor_updates'
+HOST_ROUTES = 'host_routes'
 ID = 'id'
 IMAGE_ID = 'image_id'
 IP_ADDRESS = 'ip_address'
@@ -367,6 +371,7 @@ LOADBALANCER = 'loadbalancer'
 LOADBALANCER_ID = 'loadbalancer_id'
 LOAD_BALANCER_ID = 'load_balancer_id'
 LOAD_BALANCER_UPDATES = 'load_balancer_updates'
+MAC_ADDRESS = 'mac_address'
 MANAGEMENT_NETWORK = 'management_network'
 MEMBER = 'member'
 MEMBER_ID = 'member_id'
@@ -376,6 +381,7 @@ MESSAGE = 'message'
 NAME = 'name'
 NETWORK = 'network'
 NETWORK_ID = 'network_id'
+NEXTHOP = 'nexthop'
 NICS = 'nics'
 OBJECT = 'object'
 ORIGINAL_HEALTH_MONITOR = 'original_health_monitor'
@@ -423,10 +429,12 @@ TOPOLOGY = 'topology'
 TOTAL_CONNECTIONS = 'total_connections'
 UPDATED_AT = 'updated_at'
 UPDATE_DICT = 'update_dict'
+UPDATED_PORTS = 'updated_ports'
 VALID_VIP_NETWORKS = 'valid_vip_networks'
 VIP = 'vip'
 VIP_ADDRESS = 'vip_address'
 VIP_NETWORK = 'vip_network'
+VIP_NETWORK_ID = 'vip_network_id'
 VIP_PORT_ID = 'vip_port_id'
 VIP_QOS_POLICY_ID = 'vip_qos_policy_id'
 VIP_SG_ID = 'vip_sg_id'
@@ -579,7 +587,6 @@ RPC_NAMESPACE_CONTROLLER_AGENT = 'controller'
 # Build Type Priority
 LB_CREATE_FAILOVER_PRIORITY = 20
 LB_CREATE_NORMAL_PRIORITY = 40
-LB_CREATE_SPARES_POOL_PRIORITY = 60
 LB_CREATE_ADMIN_FAILOVER_PRIORITY = 80
 BUILD_TYPE_PRIORITY = 'build_type_priority'
 
@@ -782,6 +789,7 @@ RBAC_ROLES_DEPRECATED_REASON = (
 # PROVIDERS
 OCTAVIA = 'octavia'
 AMPHORAV2 = 'amphorav2'
+AMPHORAV1 = 'amphorav1'
 
 # systemctl commands
 DISABLE = 'disable'
@@ -792,6 +800,7 @@ AMP_NETNS_SVC_PREFIX = 'amphora-netns'
 
 # Amphora Feature Compatibility
 HTTP_REUSE = 'has_http_reuse'
+SERVER_STATE_FILE = 'has_server_state_file'
 POOL_ALPN = 'has_pool_alpn'
 
 # TODO(johnsom) convert these to octavia_lib constants
@@ -925,5 +934,8 @@ IFACE_DOWN = 'down'
 
 COMMAND = 'command'
 
+IFLA_ADDRESS = 'IFLA_ADDRESS'
+IFLA_IFNAME = 'IFLA_IFNAME'
+
 # Amphora network directory
 AMP_NET_DIR_TEMPLATE = '/etc/octavia/interfaces/'
