@@ -45,9 +45,8 @@ class LoadBalancerFlows(object):
         self.member_flows = member_flows.MemberFlows()
 
     def get_create_load_balancer_flow(self, topology, listeners=None):
-        """Creates a conditional graph flow that allocates a loadbalancer to
+        """Creates a conditional graph flow that allocates a loadbalancer.
 
-        two spare amphorae.
         :raises InvalidTopology: Invalid topology specified
         :return: The graph flow for creating a loadbalancer.
         """
@@ -203,12 +202,12 @@ class LoadBalancerFlows(object):
         )
         flows.append(
             network_tasks.HandleNetworkDeltas(
-                requires=constants.DELTAS, provides=constants.ADDED_PORTS
+                requires=constants.DELTAS, provides=constants.UPDATED_PORTS
             )
         )
         flows.append(
             amphora_driver_tasks.AmphoraePostNetworkPlug(
-                requires=(constants.LOADBALANCER, constants.ADDED_PORTS)
+                requires=(constants.LOADBALANCER, constants.UPDATED_PORTS)
             )
         )
         flows.append(
@@ -400,8 +399,6 @@ class LoadBalancerFlows(object):
                 amp_role = 'master_or_backup'
             elif failed_amp.role == constants.ROLE_STANDALONE:
                 amp_role = 'standalone'
-            elif failed_amp.role is None:
-                amp_role = 'spare'
             else:
                 amp_role = 'undefined'
             LOG.info("Performing failover for amphora: %s",
@@ -547,8 +544,6 @@ class LoadBalancerFlows(object):
                     amp_role = 'master_or_backup'
                 elif failed_amp.role == constants.ROLE_STANDALONE:
                     amp_role = 'standalone'
-                elif failed_amp.role is None:
-                    amp_role = 'spare'
                 else:
                     amp_role = 'undefined'
                 LOG.info("Performing failover for amphora: %s",
