@@ -26,9 +26,8 @@ from taskflow.types import failure
 
 from octavia.common import constants
 from octavia.common import data_models
-import octavia.common.tls_utils.cert_parser as cert_parser
+from octavia.common.tls_utils import cert_parser
 from octavia.common import utils
-from octavia.common import validate
 from octavia.controller.worker import task_utils as task_utilities
 from octavia.db import api as db_apis
 from octavia.db import repositories as repo
@@ -53,7 +52,7 @@ class BaseDatabaseTask(task.Task):
         self.l7policy_repo = repo.L7PolicyRepository()
         self.l7rule_repo = repo.L7RuleRepository()
         self.task_utils = task_utilities.TaskUtils()
-        super(BaseDatabaseTask, self).__init__(**kwargs)
+        super().__init__(**kwargs)
 
     def _delete_from_amp_health(self, amphora_id):
         """Delete the amphora_health record for an amphora.
@@ -195,7 +194,7 @@ class DeleteHealthMonitorInDBByPool(DeleteHealthMonitorInDB):
         :param pool: A pool which health monitor should be deleted.
         :returns: None
         """
-        super(DeleteHealthMonitorInDBByPool, self).execute(
+        super().execute(
             pool.health_monitor)
 
     def revert(self, pool, *args, **kwargs):
@@ -204,7 +203,7 @@ class DeleteHealthMonitorInDBByPool(DeleteHealthMonitorInDB):
         :param pool: A pool which health monitor couldn't be deleted
         :returns: None
         """
-        super(DeleteHealthMonitorInDBByPool, self).revert(
+        super().revert(
             pool.health_monitor, *args, **kwargs)
 
 
@@ -497,66 +496,6 @@ class AssociateFailoverAmphoraWithLBID(BaseDatabaseTask):
                       "%(except)s", {'amp': amphora_id, 'except': str(e)})
 
 
-class MapLoadbalancerToAmphora(BaseDatabaseTask):
-    """Maps and assigns a load balancer to an amphora in the database."""
-
-    def execute(self, loadbalancer_id, server_group_id=None, flavor=None,
-                availability_zone=None):
-        """Allocates an Amphora for the load balancer in the database.
-
-        :param loadbalancer_id: The load balancer id to map to an amphora
-        :returns: Amphora ID if one was allocated, None if it was
-                  unable to allocate an Amphora
-        """
-
-        LOG.debug("Allocating an Amphora for load balancer with id %s",
-                  loadbalancer_id)
-
-        if server_group_id is not None:
-            LOG.debug("Load balancer is using anti-affinity. Skipping spares "
-                      "pool allocation.")
-            return None
-
-        # Validate the flavor is spares compatible
-        if not validate.is_flavor_spares_compatible(flavor):
-            LOG.debug("Load balancer has a flavor that is not compatible with "
-                      "using spares pool amphora. Skipping spares pool "
-                      "allocation.")
-            return None
-
-        if availability_zone:
-            amp_az = availability_zone.get(constants.COMPUTE_ZONE)
-        else:
-            amp_az = CONF.nova.availability_zone
-
-        try:
-            amp = self.amphora_repo.allocate_and_associate(
-                db_apis.get_session(),
-                loadbalancer_id,
-                amp_az)
-        except Exception as e:
-            LOG.error("Failed to get a spare amphora (AZ: %(amp_az)s) for "
-                      "loadbalancer %(lb_id)s due to: %(except)s",
-                      {'amp_az': amp_az, 'lb_id': loadbalancer_id,
-                       'except': str(e)})
-            return None
-
-        if amp is None:
-            LOG.debug("No Amphora available for load balancer with id %s",
-                      loadbalancer_id)
-            return None
-
-        LOG.debug("Allocated Amphora with id %(amp)s for load balancer "
-                  "with id %(lb)s", {'amp': amp.id, 'lb': loadbalancer_id})
-
-        return amp.id
-
-    def revert(self, result, loadbalancer_id, *args, **kwargs):
-        LOG.warning("Reverting Amphora allocation for the load "
-                    "balancer %s in the database.", loadbalancer_id)
-        self.task_utils.mark_loadbalancer_prov_status_error(loadbalancer_id)
-
-
 class _MarkAmphoraRoleAndPriorityInDB(BaseDatabaseTask):
     """Alter the amphora role and priority in DB."""
 
@@ -969,7 +908,7 @@ class MarkLBActiveInDB(BaseDatabaseTask):
     """
 
     def __init__(self, mark_subobjects=False, **kwargs):
-        super(MarkLBActiveInDB, self).__init__(**kwargs)
+        super().__init__(**kwargs)
         self.mark_subobjects = mark_subobjects
 
     def execute(self, loadbalancer):
@@ -1070,10 +1009,6 @@ class MarkLBActiveInDB(BaseDatabaseTask):
                     LOG.warning("Error updating listener %s provisioning "
                                 "status", listener.id)
 
-        LOG.warning("Reverting mark load balancer deleted in DB "
-                    "for load balancer id %s", loadbalancer.id)
-        self.task_utils.mark_loadbalancer_prov_status_error(loadbalancer.id)
-
 
 class UpdateLBServerGroupInDB(BaseDatabaseTask):
     """Update the server group id info for load balancer in DB."""
@@ -1133,17 +1068,6 @@ class MarkLBDeletedInDB(BaseDatabaseTask):
                                       loadbalancer.id,
                                       provisioning_status=constants.DELETED)
 
-    def revert(self, loadbalancer, *args, **kwargs):
-        """Mark the load balancer as broken and ready to be cleaned up.
-
-        :param loadbalancer: Load balancer object that failed to update
-        :returns: None
-        """
-
-        LOG.warning("Reverting mark load balancer deleted in DB "
-                    "for load balancer id %s", loadbalancer.id)
-        self.task_utils.mark_loadbalancer_prov_status_error(loadbalancer.id)
-
 
 class MarkLBPendingDeleteInDB(BaseDatabaseTask):
     """Mark the load balancer pending delete in the DB.
@@ -1165,17 +1089,6 @@ class MarkLBPendingDeleteInDB(BaseDatabaseTask):
                                       provisioning_status=(constants.
                                                            PENDING_DELETE))
 
-    def revert(self, loadbalancer, *args, **kwargs):
-        """Mark the load balancer as broken and ready to be cleaned up.
-
-        :param loadbalancer: Load balancer object that failed to update
-        :returns: None
-        """
-
-        LOG.warning("Reverting mark load balancer pending delete in DB "
-                    "for load balancer id %s", loadbalancer.id)
-        self.task_utils.mark_loadbalancer_prov_status_error(loadbalancer.id)
-
 
 class MarkLBAndListenersActiveInDB(BaseDatabaseTask):
     """Mark the load balancer and specified listeners active in the DB.
@@ -1209,12 +1122,10 @@ class MarkLBAndListenersActiveInDB(BaseDatabaseTask):
         :returns: None
         """
 
-        LOG.warning("Reverting mark load balancer and listeners active in DB "
-                    "for load balancer id %(LB)s and listener ids: %(list)s",
-                    {'LB': loadbalancer.id,
-                     'list': ', '.join([listener.id
+        LOG.warning("Reverting mark listeners active in DB "
+                    "for listener ids: %(list)s",
+                    {'list': ', '.join([listener.id
                                         for listener in listeners])})
-        self.task_utils.mark_loadbalancer_prov_status_error(loadbalancer.id)
         for listener in listeners:
             self.task_utils.mark_listener_prov_status_error(listener.id)
 
@@ -1301,18 +1212,6 @@ class UpdateLoadbalancerInDB(BaseDatabaseTask):
         self.loadbalancer_repo.update(db_apis.get_session(), loadbalancer.id,
                                       **update_dict)
 
-    def revert(self, loadbalancer, *args, **kwargs):
-        """Mark the loadbalancer ERROR since the update couldn't happen
-
-        :param loadbalancer: The load balancer that couldn't be updated
-        :returns: None
-        """
-
-        LOG.warning("Reverting update loadbalancer in DB "
-                    "for loadbalancer id %s", loadbalancer.id)
-
-        self.task_utils.mark_loadbalancer_prov_status_error(loadbalancer.id)
-
 
 class UpdateHealthMonInDB(BaseDatabaseTask):
     """Update the health monitor in the DB.
@@ -2689,6 +2588,142 @@ class CountPoolChildrenForQuota(BaseDatabaseTask):
         return {'HM': health_mon_count, 'member': member_count}
 
 
+class DecrementL7policyQuota(BaseDatabaseTask):
+    """Decrements the l7policy quota for a project.
+
+    Since sqlalchemy will likely retry by itself always revert if it fails
+    """
+
+    def execute(self, l7policy):
+        """Decrements the l7policy quota.
+
+        :param l7policy: The l7policy to decrement the quota on.
+        :returns: None
+        """
+
+        LOG.debug("Decrementing l7policy quota for "
+                  "project: %s ", l7policy.project_id)
+
+        lock_session = db_apis.get_session(autocommit=False)
+        try:
+            self.repos.decrement_quota(lock_session,
+                                       data_models.L7Policy,
+                                       l7policy.project_id)
+
+            if l7policy.l7rules:
+                self.repos.decrement_quota(lock_session,
+                                           data_models.L7Rule,
+                                           l7policy.project_id,
+                                           quantity=len(l7policy.l7rules))
+            lock_session.commit()
+        except Exception:
+            with excutils.save_and_reraise_exception():
+                LOG.error('Failed to decrement l7policy quota for project: '
+                          '%(proj)s the project may have excess quota in use.',
+                          {'proj': l7policy.project_id})
+                lock_session.rollback()
+
+    def revert(self, l7policy, result, *args, **kwargs):
+        """Re-apply the quota
+
+        :param l7policy: The l7policy to decrement the quota on.
+        :returns: None
+        """
+
+        LOG.warning('Reverting decrement quota for l7policy on project'
+                    ' %(proj)s Project quota counts may be incorrect.',
+                    {'proj': l7policy.project_id})
+
+        # Increment the quota back if this task wasn't the failure
+        if not isinstance(result, failure.Failure):
+
+            try:
+                session = db_apis.get_session()
+                lock_session = db_apis.get_session(autocommit=False)
+                try:
+                    self.repos.check_quota_met(session,
+                                               lock_session,
+                                               data_models.L7Policy,
+                                               l7policy.project_id)
+                    lock_session.commit()
+                except Exception:
+                    lock_session.rollback()
+
+                # Attempt to increment back the L7Rule quota
+                for i in range(len(l7policy.l7rules)):
+                    lock_session = db_apis.get_session(autocommit=False)
+                    try:
+                        self.repos.check_quota_met(session,
+                                                   lock_session,
+                                                   data_models.L7Rule,
+                                                   l7policy.project_id)
+                        lock_session.commit()
+                    except Exception:
+                        lock_session.rollback()
+            except Exception:
+                # Don't fail the revert flow
+                pass
+
+
+class DecrementL7ruleQuota(BaseDatabaseTask):
+    """Decrements the l7rule quota for a project.
+
+    Since sqlalchemy will likely retry by itself always revert if it fails
+    """
+
+    def execute(self, l7rule):
+        """Decrements the l7rule quota.
+
+        :param l7rule: The l7rule to decrement the quota on.
+        :returns: None
+        """
+
+        LOG.debug("Decrementing l7rule quota for "
+                  "project: %s ", l7rule.project_id)
+
+        lock_session = db_apis.get_session(autocommit=False)
+        try:
+            self.repos.decrement_quota(lock_session,
+                                       data_models.L7Rule,
+                                       l7rule.project_id)
+            lock_session.commit()
+        except Exception:
+            with excutils.save_and_reraise_exception():
+                LOG.error('Failed to decrement l7rule quota for project: '
+                          '%(proj)s the project may have excess quota in use.',
+                          {'proj': l7rule.project_id})
+                lock_session.rollback()
+
+    def revert(self, l7rule, result, *args, **kwargs):
+        """Re-apply the quota
+
+        :param l7rule: The l7rule to decrement the quota on.
+        :returns: None
+        """
+
+        LOG.warning('Reverting decrement quota for l7rule on project %(proj)s '
+                    'Project quota counts may be incorrect.',
+                    {'proj': l7rule.project_id})
+
+        # Increment the quota back if this task wasn't the failure
+        if not isinstance(result, failure.Failure):
+
+            try:
+                session = db_apis.get_session()
+                lock_session = db_apis.get_session(autocommit=False)
+                try:
+                    self.repos.check_quota_met(session,
+                                               lock_session,
+                                               data_models.L7Rule,
+                                               l7rule.project_id)
+                    lock_session.commit()
+                except Exception:
+                    lock_session.rollback()
+            except Exception:
+                # Don't fail the revert flow
+                pass
+
+
 class UpdatePoolMembersOperatingStatusInDB(BaseDatabaseTask):
     """Updates the members of a pool operating status.
 
