@@ -156,11 +156,8 @@ class ControllerWorker(object):
         pool.health_monitor = db_health_monitor
         load_balancer = pool.load_balancer
         provider_lb = provider_utils.db_loadbalancer_to_provider_loadbalancer(
-            load_balancer).to_dict()
-
-        listeners_dicts = (
-            provider_utils.db_listeners_to_provider_dicts_list_of_dicts(
-                pool.listeners))
+            load_balancer).to_dict(recurse=True)
+        listeners_dicts = provider_lb.get('listeners', [])
 
         store = {constants.HEALTH_MON: health_monitor,
                  constants.POOL_ID: pool.id,
@@ -185,11 +182,8 @@ class ControllerWorker(object):
         pool = db_health_monitor.pool
         load_balancer = pool.load_balancer
         provider_lb = provider_utils.db_loadbalancer_to_provider_loadbalancer(
-            load_balancer).to_dict()
-
-        listeners_dicts = (
-            provider_utils.db_listeners_to_provider_dicts_list_of_dicts(
-                pool.listeners))
+            load_balancer).to_dict(recurse=True)
+        listeners_dicts = provider_lb.get('listeners', [])
 
         store = {constants.HEALTH_MON: health_monitor,
                  constants.POOL_ID: pool.id,
@@ -224,13 +218,10 @@ class ControllerWorker(object):
 
         pool = db_health_monitor.pool
 
-        listeners_dicts = (
-            provider_utils.db_listeners_to_provider_dicts_list_of_dicts(
-                pool.listeners))
-
         load_balancer = pool.load_balancer
         provider_lb = provider_utils.db_loadbalancer_to_provider_loadbalancer(
-            load_balancer).to_dict()
+            load_balancer).to_dict(recurse=True)
+        listeners_dicts = provider_lb.get('listeners', [])
 
         store = {constants.HEALTH_MON: original_health_monitor,
                  constants.POOL_ID: pool.id,
@@ -266,15 +257,10 @@ class ControllerWorker(object):
             raise db_exceptions.NoResultFound
 
         load_balancer = db_listener.load_balancer
-        listeners = load_balancer.listeners
-        dict_listeners = []
-        for li in listeners:
-            dict_listeners.append(
-                provider_utils.db_listener_to_provider_listener(li).to_dict())
         provider_lb = provider_utils.db_loadbalancer_to_provider_loadbalancer(
-            load_balancer).to_dict()
+            load_balancer).to_dict(recurse=True)
 
-        store = {constants.LISTENERS: dict_listeners,
+        store = {constants.LISTENERS: provider_lb['listeners'],
                  constants.LOADBALANCER: provider_lb,
                  constants.LOADBALANCER_ID: load_balancer.id}
 
@@ -305,8 +291,17 @@ class ControllerWorker(object):
         :returns: None
         :raises ListenerNotFound: The referenced listener was not found
         """
-        db_lb = self._lb_repo.get(db_apis.get_session(),
-                                  id=listener[constants.LOADBALANCER_ID])
+        try:
+            db_lb = self._get_db_obj_until_pending_update(
+                self._lb_repo, listener[constants.LOADBALANCER_ID])
+        except tenacity.RetryError as e:
+            LOG.warning('Loadbalancer did not go into %s in 60 seconds. '
+                        'This either due to an in-progress Octavia upgrade '
+                        'or an overloaded and failing database. Assuming '
+                        'an upgrade is in progress and continuing.',
+                        constants.PENDING_UPDATE)
+            db_lb = e.last_attempt.result()
+
         store = {constants.LISTENER: listener,
                  constants.UPDATE_DICT: listener_updates,
                  constants.LOADBALANCER_ID: db_lb.id,
@@ -402,6 +397,18 @@ class ControllerWorker(object):
         :returns: None
         :raises LBNotFound: The referenced load balancer was not found
         """
+
+        try:
+            self._get_db_obj_until_pending_update(
+                self._lb_repo,
+                original_load_balancer[constants.LOADBALANCER_ID])
+        except tenacity.RetryError:
+            LOG.warning('Load balancer did not go into %s in 60 seconds. '
+                        'This either due to an in-progress Octavia upgrade '
+                        'or an overloaded and failing database. Assuming '
+                        'an upgrade is in progress and continuing.',
+                        constants.PENDING_UPDATE)
+
         store = {constants.LOADBALANCER: original_load_balancer,
                  constants.LOADBALANCER_ID:
                      original_load_balancer[constants.LOADBALANCER_ID],
@@ -437,11 +444,8 @@ class ControllerWorker(object):
         pool = db_member.pool
         load_balancer = pool.load_balancer
         provider_lb = provider_utils.db_loadbalancer_to_provider_loadbalancer(
-            load_balancer).to_dict()
-
-        listeners_dicts = (
-            provider_utils.db_listeners_to_provider_dicts_list_of_dicts(
-                pool.listeners))
+            load_balancer).to_dict(recurse=True)
+        listeners_dicts = provider_lb.get('listeners', [])
 
         store = {
             constants.MEMBER: member,
@@ -472,11 +476,8 @@ class ControllerWorker(object):
 
         load_balancer = pool.load_balancer
         provider_lb = provider_utils.db_loadbalancer_to_provider_loadbalancer(
-            load_balancer).to_dict()
-
-        listeners_dicts = (
-            provider_utils.db_listeners_to_provider_dicts_list_of_dicts(
-                pool.listeners))
+            load_balancer).to_dict(recurse=True)
+        listeners_dicts = provider_lb.get('listeners', [])
 
         store = {
             constants.MEMBER: member,
@@ -496,8 +497,26 @@ class ControllerWorker(object):
             flow_utils.get_delete_member_flow,
             store=store)
 
+    @tenacity.retry(
+        retry=tenacity.retry_if_exception_type(db_exceptions.NoResultFound),
+        wait=tenacity.wait_incrementing(
+            CONF.haproxy_amphora.api_db_commit_retry_initial_delay,
+            CONF.haproxy_amphora.api_db_commit_retry_backoff,
+            CONF.haproxy_amphora.api_db_commit_retry_max),
+        stop=tenacity.stop_after_attempt(
+            CONF.haproxy_amphora.api_db_commit_retry_attempts))
     def batch_update_members(self, old_members, new_members,
                              updated_members):
+        db_new_members = [self._member_repo.get(db_apis.get_session(),
+                                                id=member[constants.MEMBER_ID])
+                          for member in new_members]
+        # The API may not have commited all of the new member records yet.
+        # Make sure we retry looking them up.
+        if None in db_new_members or len(db_new_members) != len(new_members):
+            LOG.warning('Failed to fetch one of the new members from DB. '
+                        'Retrying for up to 60 seconds.')
+            raise db_exceptions.NoResultFound
+
         updated_members = [
             (provider_utils.db_member_to_provider_member(
                 self._member_repo.get(db_apis.get_session(),
@@ -521,11 +540,9 @@ class ControllerWorker(object):
                 id=updated_members[0][0][constants.POOL_ID])
         load_balancer = pool.load_balancer
 
-        listeners_dicts = (
-            provider_utils.db_listeners_to_provider_dicts_list_of_dicts(
-                pool.listeners))
         provider_lb = provider_utils.db_loadbalancer_to_provider_loadbalancer(
-            load_balancer).to_dict()
+            load_balancer).to_dict(recurse=True)
+        listeners_dicts = provider_lb.get('listeners', [])
 
         store = {
             constants.LISTENERS: listeners_dicts,
@@ -553,16 +570,23 @@ class ControllerWorker(object):
         :returns: None
         :raises MemberNotFound: The referenced member was not found
         """
-        # TODO(ataraday) when other flows will use dicts - revisit this
-        pool = self._pool_repo.get(db_apis.get_session(),
-                                   id=member[constants.POOL_ID])
+
+        try:
+            db_member = self._get_db_obj_until_pending_update(
+                self._member_repo, member[constants.MEMBER_ID])
+        except tenacity.RetryError as e:
+            LOG.warning('Member did not go into %s in 60 seconds. '
+                        'This either due to an in-progress Octavia upgrade '
+                        'or an overloaded and failing database. Assuming '
+                        'an upgrade is in progress and continuing.',
+                        constants.PENDING_UPDATE)
+            db_member = e.last_attempt.result()
+
+        pool = db_member.pool
         load_balancer = pool.load_balancer
         provider_lb = provider_utils.db_loadbalancer_to_provider_loadbalancer(
-            load_balancer).to_dict()
-
-        listeners_dicts = (
-            provider_utils.db_listeners_to_provider_dicts_list_of_dicts(
-                pool.listeners))
+            load_balancer).to_dict(recurse=True)
+        listeners_dicts = provider_lb.get('listeners', [])
         store = {
             constants.MEMBER: member,
             constants.LISTENERS: listeners_dicts,
@@ -608,11 +632,8 @@ class ControllerWorker(object):
 
         load_balancer = db_pool.load_balancer
         provider_lb = provider_utils.db_loadbalancer_to_provider_loadbalancer(
-            load_balancer).to_dict()
-
-        listeners_dicts = (
-            provider_utils.db_listeners_to_provider_dicts_list_of_dicts(
-                db_pool.listeners))
+            load_balancer).to_dict(recurse=True)
+        listeners_dicts = provider_lb.get('listeners', [])
 
         store = {constants.POOL_ID: pool[constants.POOL_ID],
                  constants.LISTENERS: listeners_dicts,
@@ -632,13 +653,10 @@ class ControllerWorker(object):
         db_pool = self._pool_repo.get(db_apis.get_session(),
                                       id=pool[constants.POOL_ID])
 
-        listeners_dicts = (
-            provider_utils.db_listeners_to_provider_dicts_list_of_dicts(
-                db_pool.listeners))
         load_balancer = db_pool.load_balancer
-
         provider_lb = provider_utils.db_loadbalancer_to_provider_loadbalancer(
-            load_balancer).to_dict()
+            load_balancer).to_dict(recurse=True)
+        listeners_dicts = provider_lb.get('listeners', [])
 
         store = {constants.POOL_ID: pool[constants.POOL_ID],
                  constants.LISTENERS: listeners_dicts,
@@ -670,11 +688,8 @@ class ControllerWorker(object):
 
         load_balancer = db_pool.load_balancer
         provider_lb = provider_utils.db_loadbalancer_to_provider_loadbalancer(
-            load_balancer).to_dict()
-
-        listeners_dicts = (
-            provider_utils.db_listeners_to_provider_dicts_list_of_dicts(
-                db_pool.listeners))
+            load_balancer).to_dict(recurse=True)
+        listeners_dicts = provider_lb.get('listeners', [])
 
         store = {constants.POOL_ID: db_pool.id,
                  constants.LISTENERS: listeners_dicts,
@@ -751,8 +766,18 @@ class ControllerWorker(object):
         :returns: None
         :raises L7PolicyNotFound: The referenced l7policy was not found
         """
-        db_listener = self._listener_repo.get(
-            db_apis.get_session(), id=original_l7policy[constants.LISTENER_ID])
+        try:
+            db_l7policy = self._get_db_obj_until_pending_update(
+                self._l7policy_repo, original_l7policy[constants.L7POLICY_ID])
+        except tenacity.RetryError as e:
+            LOG.warning('L7 policy did not go into %s in 60 seconds. '
+                        'This either due to an in-progress Octavia upgrade '
+                        'or an overloaded and failing database. Assuming '
+                        'an upgrade is in progress and continuing.',
+                        constants.PENDING_UPDATE)
+            db_l7policy = e.last_attempt.result()
+
+        db_listener = db_l7policy.listener
 
         listeners_dicts = (
             provider_utils.db_listeners_to_provider_dicts_list_of_dicts(
@@ -843,8 +868,17 @@ class ControllerWorker(object):
         :returns: None
         :raises L7RuleNotFound: The referenced l7rule was not found
         """
-        db_l7policy = self._l7policy_repo.get(
-            db_apis.get_session(), id=original_l7rule[constants.L7POLICY_ID])
+        try:
+            db_l7rule = self._get_db_obj_until_pending_update(
+                self._l7rule_repo, original_l7rule[constants.L7RULE_ID])
+        except tenacity.RetryError as e:
+            LOG.warning('L7 rule did not go into %s in 60 seconds. '
+                        'This either due to an in-progress Octavia upgrade '
+                        'or an overloaded and failing database. Assuming '
+                        'an upgrade is in progress and continuing.',
+                        constants.PENDING_UPDATE)
+            db_l7rule = e.last_attempt.result()
+        db_l7policy = db_l7rule.l7policy
         load_balancer = db_l7policy.listener.load_balancer
 
         listeners_dicts = (
@@ -948,7 +982,7 @@ class ControllerWorker(object):
             self.run_flow(
                 flow_utils.get_failover_amphora_flow,
                 amphora.to_dict(), lb_amp_count,
-                store=stored_params, wait=True)
+                store=stored_params)
 
             LOG.info("Successfully completed the failover for an amphora: %s",
                      {"id": amphora_id,
@@ -1099,7 +1133,7 @@ class ControllerWorker(object):
 
             self.run_flow(
                 flow_utils.get_failover_LB_flow, amps, provider_lb_dict,
-                store=stored_params, wait=True)
+                store=stored_params)
 
             LOG.info('Failover of load balancer %s completed successfully.',
                      lb.id)
