@@ -193,13 +193,24 @@ class TestPlug(base.TestCase):
         mock_webob.Response.assert_any_call(json={'message': 'Invalid VIP'},
                                             status=400)
 
+    @mock.patch("octavia.amphorae.backends.agent.api_server.osutils."
+                "BaseOS.write_interface_file")
+    def test_plug_lo(self, mock_write_interface):
+        m = mock.mock_open()
+        with mock.patch('os.open'), mock.patch.object(os, 'fdopen', m):
+            self.test_plug.plug_lo()
+        mock_write_interface.assert_called_once_with(interface='lo',
+                                                     ip_address='127.0.0.1',
+                                                     prefixlen=8)
+
     @mock.patch('pyroute2.NetNS', create=True)
     def test__netns_interface_exists(self, mock_netns):
 
         netns_handle = mock_netns.return_value.__enter__.return_value
 
         netns_handle.get_links.return_value = [{
-            'attrs': [['IFLA_ADDRESS', '123']]}]
+            'attrs': [['IFLA_ADDRESS', '123'],
+                      ['IFLA_IFNAME', 'eth0']]}]
 
         # Interface is found in netns
         self.assertTrue(self.test_plug._netns_interface_exists('123'))
@@ -207,141 +218,141 @@ class TestPlug(base.TestCase):
         # Interface is not found in netns
         self.assertFalse(self.test_plug._netns_interface_exists('321'))
 
+    @mock.patch.object(plug, "webob")
+    @mock.patch('octavia.amphorae.backends.agent.api_server.plug.Plug.'
+                '_netns_interface_exists', return_value=False)
+    @mock.patch('octavia.amphorae.backends.agent.api_server.plug.Plug.'
+                '_interface_by_mac', return_value=FAKE_INTERFACE)
+    @mock.patch('pyroute2.IPRoute', create=True)
+    @mock.patch('pyroute2.NetNS', create=True)
+    @mock.patch("octavia.amphorae.backends.agent.api_server.osutils."
+                "BaseOS.write_port_interface_file")
+    @mock.patch("octavia.amphorae.backends.agent.api_server.osutils."
+                "BaseOS.bring_interface_up")
+    def test_plug_network(self, mock_if_up, mock_write_port_interface,
+                          mock_netns, mock_iproute,
+                          mock_by_mac, mock_interface_exists, mock_webob):
+        fixed_ips = [
+            {'ip_address': FAKE_IP_IPV4,
+             'subnet_cidr': FAKE_CIDR_IPV4,
+             'gateway': FAKE_GATEWAY_IPV4,
+             'host_routes': [
+                 {'destination': '192.0.2.0/24',
+                  'nexthop': '192.0.2.254'}]
+             }]
+        mtu = 1400
+        m = mock.mock_open()
+        with mock.patch('os.open'), mock.patch.object(os, 'fdopen', m):
+            self.test_plug.plug_network(FAKE_MAC_ADDRESS, fixed_ips, 1400)
 
-class TestPlugNetwork(base.TestCase):
-    def setUp(self):
-        super().setUp()
-        self.mock_platform = mock.patch("distro.id").start()
+        mock_write_port_interface.assert_called_once_with(
+            interface='eth0', fixed_ips=fixed_ips, mtu=mtu)
+        mock_if_up.assert_called_once_with('eth0', 'network')
 
-    def __generate_network_file_text_static_ip(self):
-        netns_interface = 'eth1234'
-        FIXED_IP = '192.0.2.2'
-        BROADCAST = '192.0.2.255'
-        SUBNET_CIDR = '192.0.2.0/24'
-        NETMASK = '255.255.255.0'
-        DEST1 = '198.51.100.0/24'
-        DEST2 = '203.0.113.0/24'
-        NEXTHOP = '192.0.2.1'
-        MTU = 1450
-        fixed_ips = [{'ip_address': FIXED_IP,
-                      'subnet_cidr': SUBNET_CIDR,
-                      'host_routes': [
-                          {'destination': DEST1, 'nexthop': NEXTHOP},
-                          {'destination': DEST2, 'nexthop': NEXTHOP}
-                      ]}]
-        format_text = (
-            '\n\n# Generated by Octavia agent\n'
-            'auto {netns_interface}\n'
-            'iface {netns_interface} inet static\n'
-            'address {fixed_ip}\n'
-            'broadcast {broadcast}\n'
-            'netmask {netmask}\n'
-            'mtu {mtu}\n'
-            'up route add -net {dest1} gw {nexthop} dev {netns_interface}\n'
-            'down route del -net {dest1} gw {nexthop} dev {netns_interface}\n'
-            'up route add -net {dest2} gw {nexthop} dev {netns_interface}\n'
-            'down route del -net {dest2} gw {nexthop} dev {netns_interface}\n'
-            'post-up /usr/local/bin/lvs-masquerade.sh add ipv4 eth1234\n'
-            'post-down /usr/local/bin/lvs-masquerade.sh delete ipv4 eth1234\n')
+        mock_webob.Response.assert_any_call(
+            json={'message': 'OK',
+                  'details': 'Plugged on interface eth0'},
+            status=202)
 
-        template_port = osutils.j2_env.get_template('plug_port_ethX.conf.j2')
-        text = self.test_plug._osutils._generate_network_file_text(
-            netns_interface, fixed_ips, MTU, template_port)
-        expected_text = format_text.format(netns_interface=netns_interface,
-                                           fixed_ip=FIXED_IP,
-                                           broadcast=BROADCAST,
-                                           netmask=NETMASK,
-                                           mtu=MTU,
-                                           dest1=DEST1,
-                                           dest2=DEST2,
-                                           nexthop=NEXTHOP)
-        self.assertEqual(expected_text, text)
+    @mock.patch.object(plug, "webob")
+    @mock.patch('octavia.amphorae.backends.agent.api_server.plug.Plug.'
+                '_netns_interface_exists', return_value=True)
+    @mock.patch('octavia.amphorae.backends.agent.api_server.plug.Plug.'
+                '_netns_interface_by_mac', return_value=FAKE_INTERFACE)
+    @mock.patch('pyroute2.NetNS', create=True)
+    @mock.patch("octavia.amphorae.backends.agent.api_server.osutils."
+                "BaseOS.write_port_interface_file")
+    @mock.patch("octavia.amphorae.backends.agent.api_server.osutils."
+                "BaseOS.bring_interface_up")
+    def test_plug_network_existing_interface(self, mock_if_up,
+                                             mock_write_port_interface,
+                                             mock_netns, mock_by_mac,
+                                             mock_interface_exists,
+                                             mock_webob):
+        fixed_ips = [
+            {'ip_address': FAKE_IP_IPV4,
+             'subnet_cidr': FAKE_CIDR_IPV4,
+             'gateway': FAKE_GATEWAY_IPV4,
+             'host_routes': [
+                 {'destination': '192.0.2.0/24',
+                  'nexthop': '192.0.2.254'}]
+             }, {'ip_address': FAKE_IP_IPV6,
+                 'subnet_cidr': FAKE_CIDR_IPV6,
+                 'gateway': FAKE_GATEWAY_IPV6,
+                 'host_routes': [
+                     {'destination': '2001:db8::/64',
+                      'nexthop': '2001:db8::ffff'}]
+                 }]
+        mtu = 1400
+        m = mock.mock_open()
+        with mock.patch('os.open'), mock.patch.object(os, 'fdopen', m):
+            self.test_plug.plug_network(FAKE_MAC_ADDRESS, fixed_ips, 1400)
 
-    def __generate_network_file_text_two_static_ips(self):
-        netns_interface = 'eth1234'
-        FIXED_IP = '192.0.2.2'
-        BROADCAST = '192.0.2.255'
-        SUBNET_CIDR = '192.0.2.0/24'
-        NETMASK = '255.255.255.0'
-        DEST1 = '198.51.100.0/24'
-        DEST2 = '203.0.113.0/24'
-        NEXTHOP = '192.0.2.1'
-        MTU = 1450
-        FIXED_IP_IPV6 = '2001:0db8:0000:0000:0000:0000:0000:0001'
-        BROADCAST_IPV6 = '2001:0db8:ffff:ffff:ffff:ffff:ffff:ffff'
-        SUBNET_CIDR_IPV6 = '2001:db8::/32'
-        NETMASK_IPV6 = '32'
-        fixed_ips = [{'ip_address': FIXED_IP,
-                      'subnet_cidr': SUBNET_CIDR,
-                      'host_routes': [
-                          {'destination': DEST1, 'nexthop': NEXTHOP},
-                          {'destination': DEST2, 'nexthop': NEXTHOP}
-                      ]},
-                     {'ip_address': FIXED_IP_IPV6,
-                      'subnet_cidr': SUBNET_CIDR_IPV6,
-                      'host_routes': []}
-                     ]
-        format_text = (
-            '\n\n# Generated by Octavia agent\n'
-            'auto {netns_interface}\n'
-            'iface {netns_interface} inet static\n'
-            'address {fixed_ip}\n'
-            'broadcast {broadcast}\n'
-            'netmask {netmask}\n'
-            'mtu {mtu}\n'
-            'up route add -net {dest1} gw {nexthop} dev {netns_interface}\n'
-            'down route del -net {dest1} gw {nexthop} dev {netns_interface}\n'
-            'up route add -net {dest2} gw {nexthop} dev {netns_interface}\n'
-            'down route del -net {dest2} gw {nexthop} dev {netns_interface}\n'
-            'post-up /usr/local/bin/lvs-masquerade.sh add ipv4 '
-            '{netns_interface}\n'
-            'post-down /usr/local/bin/lvs-masquerade.sh delete ipv4 '
-            '{netns_interface}\n'
-            '\n\n# Generated by Octavia agent\n'
-            'auto {netns_interface}\n'
-            'iface {netns_interface} inet6 static\n'
-            'address {fixed_ip_ipv6}\n'
-            'broadcast {broadcast_ipv6}\n'
-            'netmask {netmask_ipv6}\n'
-            'mtu {mtu}\n'
-            'post-up /usr/local/bin/lvs-masquerade.sh add ipv6 '
-            '{netns_interface}\n'
-            'post-down /usr/local/bin/lvs-masquerade.sh delete ipv6 '
-            '{netns_interface}\n')
+        mock_write_port_interface.assert_called_once_with(
+            interface=FAKE_INTERFACE, fixed_ips=fixed_ips, mtu=mtu)
+        mock_if_up.assert_called_once_with(FAKE_INTERFACE, 'network')
 
-        template_port = osutils.j2_env.get_template('plug_port_ethX.conf.j2')
-        text = self.test_plug._osutils._generate_network_file_text(
-            netns_interface, fixed_ips, MTU, template_port)
-        expected_text = format_text.format(netns_interface=netns_interface,
-                                           fixed_ip=FIXED_IP,
-                                           broadcast=BROADCAST,
-                                           netmask=NETMASK,
-                                           mtu=MTU,
-                                           dest1=DEST1,
-                                           dest2=DEST2,
-                                           nexthop=NEXTHOP,
-                                           fixed_ip_ipv6=FIXED_IP_IPV6,
-                                           broadcast_ipv6=BROADCAST_IPV6,
-                                           netmask_ipv6=NETMASK_IPV6)
-        self.assertEqual(expected_text, text)
+        mock_webob.Response.assert_any_call(
+            json={'message': 'OK',
+                  'details': 'Updated existing interface {}'.format(
+                      FAKE_INTERFACE)},
+            status=202)
 
-    def _setup(self, os):
-        self.mock_platform.return_value = os
-        self.osutil = osutils.BaseOS.get_os_util()
-        self.test_plug = plug.Plug(self.osutil)
+    @mock.patch.object(plug, "webob")
+    @mock.patch('octavia.amphorae.backends.agent.api_server.plug.Plug.'
+                '_netns_interface_exists', return_value=True)
+    @mock.patch('octavia.amphorae.backends.agent.api_server.plug.Plug.'
+                '_netns_interface_by_mac', return_value=FAKE_INTERFACE)
+    @mock.patch('pyroute2.NetNS', create=True)
+    @mock.patch("octavia.amphorae.backends.agent.api_server.osutils."
+                "BaseOS.write_vip_interface_file")
+    @mock.patch("octavia.amphorae.backends.agent.api_server.osutils."
+                "BaseOS.bring_interface_up")
+    def test_plug_network_on_vip(
+            self, mock_if_up, mock_write_vip_interface,
+            mock_netns, mock_by_mac, mock_interface_exists, mock_webob):
+        fixed_ips = [
+            {'ip_address': FAKE_IP_IPV4,
+             'subnet_cidr': FAKE_CIDR_IPV4,
+             'gateway': FAKE_GATEWAY_IPV4,
+             'host_routes': [
+                 {'destination': '192.0.2.128/25',
+                  'nexthop': '192.0.2.100'}]
+             }, {'ip_address': FAKE_IP_IPV6,
+                 'subnet_cidr': FAKE_CIDR_IPV6,
+                 'gateway': FAKE_GATEWAY_IPV6,
+                 'host_routes': [
+                     {'destination': '2001:db8::/64',
+                      'nexthop': '2001:db8::ffff'}]
+                 }]
+        mtu = 1400
+        vip_net_info = {
+            'vip': '192.0.2.10',
+            'subnet_cidr': '192.0.2.0/25',
+            'vrrp_ip': '192.0.2.11',
+            'gateway': '192.0.2.1',
+            'host_routes': []
+        }
 
-    def test__generate_network_file_text_static_ip_ubuntu(self):
-        self._setup("ubuntu")
-        self.__generate_network_file_text_static_ip()
+        m = mock.mock_open()
+        with mock.patch('os.open'), mock.patch.object(os, 'fdopen', m):
+            self.test_plug.plug_network(FAKE_MAC_ADDRESS, fixed_ips, mtu=1400,
+                                        vip_net_info=vip_net_info)
 
-    def test__generate_network_file_text_static_ip_centos(self):
-        self._setup("centos")
-        self.__generate_network_file_text_static_ip()
+        mock_write_vip_interface.assert_called_once_with(
+            interface=FAKE_INTERFACE,
+            vip=vip_net_info['vip'],
+            ip_version=4,
+            prefixlen=25,
+            gateway=vip_net_info['gateway'],
+            vrrp_ip=vip_net_info['vrrp_ip'],
+            host_routes=[],
+            fixed_ips=fixed_ips, mtu=mtu)
 
-    def test__generate_network_file_text_two_static_ips_ubuntu(self):
-        self._setup("ubuntu")
-        self.__generate_network_file_text_two_static_ips()
+        mock_if_up.assert_called_once_with(FAKE_INTERFACE, 'vip')
 
-    def test__generate_network_file_text_two_static_ips_centos(self):
-        self._setup("centos")
-        self.__generate_network_file_text_two_static_ips()
+        mock_webob.Response.assert_any_call(
+            json={'message': 'OK',
+                  'details': 'Updated existing interface {}'.format(
+                      FAKE_INTERFACE)},
+            status=202)
