@@ -29,7 +29,7 @@ CONF = cfg.CONF
 
 class TestHaproxyCfg(base.TestCase):
     def setUp(self):
-        super(TestHaproxyCfg, self).setUp()
+        super().setUp()
         self.jinja_cfg = jinja_cfg.JinjaTemplater(
             base_amp_path='/var/lib/octavia',
             base_crt_dir='/var/lib/octavia/certs')
@@ -51,13 +51,15 @@ class TestHaproxyCfg(base.TestCase):
               "ssl crt-list {crt_list} "
               "ca-file /var/lib/octavia/certs/sample_loadbalancer_id_1/"
               "client_ca.pem verify required crl-file /var/lib/octavia/"
-              "certs/sample_loadbalancer_id_1/SHA_ID.pem ciphers {ciphers}\n"
+              "certs/sample_loadbalancer_id_1/SHA_ID.pem ciphers {ciphers} "
+              "no-sslv3 no-tlsv10 no-tlsv11 alpn {alpn}\n"
               "    mode http\n"
               "    default_backend sample_pool_id_1:sample_listener_id_1\n"
               "    timeout client 50000\n").format(
             maxconn=constants.HAPROXY_DEFAULT_MAXCONN,
             crt_list=FAKE_CRT_LIST_FILENAME,
-            ciphers=constants.CIPHERS_OWASP_SUITE_B)
+            ciphers=constants.CIPHERS_OWASP_SUITE_B,
+            alpn=",".join(constants.AMPHORA_SUPPORTED_ALPN_PROTOCOLS))
         be = ("backend sample_pool_id_1:sample_listener_id_1\n"
               "    mode http\n"
               "    balance roundrobin\n"
@@ -104,13 +106,14 @@ class TestHaproxyCfg(base.TestCase):
               "    maxconn {maxconn}\n"
               "    redirect scheme https if !{{ ssl_fc }}\n"
               "    bind 10.0.0.2:443 ssl crt-list {crt_list}"
-              "   ciphers {ciphers}\n"
+              "   ciphers {ciphers} no-sslv3 no-tlsv10 no-tlsv11 alpn {alpn}\n"
               "    mode http\n"
               "    default_backend sample_pool_id_1:sample_listener_id_1\n"
               "    timeout client 50000\n").format(
             maxconn=constants.HAPROXY_DEFAULT_MAXCONN,
             crt_list=FAKE_CRT_LIST_FILENAME,
-            ciphers=constants.CIPHERS_OWASP_SUITE_B)
+            ciphers=constants.CIPHERS_OWASP_SUITE_B,
+            alpn=",".join(constants.AMPHORA_SUPPORTED_ALPN_PROTOCOLS))
         be = ("backend sample_pool_id_1:sample_listener_id_1\n"
               "    mode http\n"
               "    balance roundrobin\n"
@@ -153,12 +156,14 @@ class TestHaproxyCfg(base.TestCase):
         fe = ("frontend sample_listener_id_1\n"
               "    maxconn {maxconn}\n"
               "    redirect scheme https if !{{ ssl_fc }}\n"
-              "    bind 10.0.0.2:443 ssl crt-list {crt_list}\n"
+              "    bind 10.0.0.2:443 ssl crt-list {crt_list}    "
+              "no-sslv3 no-tlsv10 no-tlsv11 alpn {alpn}\n"
               "    mode http\n"
               "    default_backend sample_pool_id_1:sample_listener_id_1\n"
               "    timeout client 50000\n").format(
             maxconn=constants.HAPROXY_DEFAULT_MAXCONN,
-            crt_list=FAKE_CRT_LIST_FILENAME)
+            crt_list=FAKE_CRT_LIST_FILENAME,
+            alpn=",".join(constants.AMPHORA_SUPPORTED_ALPN_PROTOCOLS))
         be = ("backend sample_pool_id_1:sample_listener_id_1\n"
               "    mode http\n"
               "    balance roundrobin\n"
@@ -192,6 +197,219 @@ class TestHaproxyCfg(base.TestCase):
                 frontend=fe, backend=be),
             rendered_obj)
 
+    def test_render_template_tls_no_versions(self):
+        conf = oslo_fixture.Config(cfg.CONF)
+        conf.config(group="haproxy_amphora", base_cert_dir='/fake_cert_dir')
+        FAKE_CRT_LIST_FILENAME = os.path.join(
+            CONF.haproxy_amphora.base_cert_dir,
+            'sample_loadbalancer_id_1/sample_listener_id_1.pem')
+        fe = ("frontend sample_listener_id_1\n"
+              "    maxconn {maxconn}\n"
+              "    redirect scheme https if !{{ ssl_fc }}\n"
+              "    bind 10.0.0.2:443 "
+              "ssl crt-list {crt_list} "
+              "ca-file /var/lib/octavia/certs/sample_loadbalancer_id_1/"
+              "client_ca.pem verify required crl-file /var/lib/octavia/"
+              "certs/sample_loadbalancer_id_1/SHA_ID.pem ciphers {ciphers} "
+              "alpn {alpn}\n"
+              "    mode http\n"
+              "    default_backend sample_pool_id_1:sample_listener_id_1\n"
+              "    timeout client 50000\n").format(
+            maxconn=constants.HAPROXY_DEFAULT_MAXCONN,
+            crt_list=FAKE_CRT_LIST_FILENAME,
+            ciphers=constants.CIPHERS_OWASP_SUITE_B,
+            alpn=",".join(constants.AMPHORA_SUPPORTED_ALPN_PROTOCOLS))
+        be = ("backend sample_pool_id_1:sample_listener_id_1\n"
+              "    mode http\n"
+              "    balance roundrobin\n"
+              "    cookie SRV insert indirect nocache\n"
+              "    timeout check 31s\n"
+              "    option httpchk GET /index.html HTTP/1.0\\r\\n\n"
+              "    http-check expect rstatus 418\n"
+              "    fullconn {maxconn}\n"
+              "    option allbackups\n"
+              "    timeout connect 5000\n"
+              "    timeout server 50000\n"
+              "    server sample_member_id_1 10.0.0.99:82 "
+              "weight 13 check inter 30s fall 3 rise 2 "
+              "cookie sample_member_id_1\n"
+              "    server sample_member_id_2 10.0.0.98:82 "
+              "weight 13 check inter 30s fall 3 rise 2 cookie "
+              "sample_member_id_2\n\n").format(
+            maxconn=constants.HAPROXY_DEFAULT_MAXCONN)
+        tls_tupe = {'cont_id_1':
+                    sample_configs_combined.sample_tls_container_tuple(
+                        id='tls_container_id',
+                        certificate='imaCert1', private_key='imaPrivateKey1',
+                        primary_cn='FakeCN'),
+                    'cont_id_ca': 'client_ca.pem',
+                    'cont_id_crl': 'SHA_ID.pem'}
+        rendered_obj = self.jinja_cfg.render_loadbalancer_obj(
+            sample_configs_combined.sample_amphora_tuple(),
+            [sample_configs_combined.sample_listener_tuple(
+                proto='TERMINATED_HTTPS', tls=True, sni=True,
+                client_ca_cert=True, client_crl_cert=True, tls_versions=None)],
+            tls_tupe)
+        self.assertEqual(
+            sample_configs_combined.sample_base_expected_config(
+                frontend=fe, backend=be),
+            rendered_obj)
+
+    def test_render_template_tls_no_ciphers_or_versions(self):
+        conf = oslo_fixture.Config(cfg.CONF)
+        conf.config(group="haproxy_amphora", base_cert_dir='/fake_cert_dir')
+        FAKE_CRT_LIST_FILENAME = os.path.join(
+            CONF.haproxy_amphora.base_cert_dir,
+            'sample_loadbalancer_id_1/sample_listener_id_1.pem')
+        fe = ("frontend sample_listener_id_1\n"
+              "    maxconn {maxconn}\n"
+              "    redirect scheme https if !{{ ssl_fc }}\n"
+              "    bind 10.0.0.2:443 ssl crt-list {crt_list}    "
+              "alpn {alpn}\n"
+              "    mode http\n"
+              "    default_backend sample_pool_id_1:sample_listener_id_1\n"
+              "    timeout client 50000\n").format(
+            maxconn=constants.HAPROXY_DEFAULT_MAXCONN,
+            crt_list=FAKE_CRT_LIST_FILENAME,
+            alpn=",".join(constants.AMPHORA_SUPPORTED_ALPN_PROTOCOLS))
+        be = ("backend sample_pool_id_1:sample_listener_id_1\n"
+              "    mode http\n"
+              "    balance roundrobin\n"
+              "    cookie SRV insert indirect nocache\n"
+              "    timeout check 31s\n"
+              "    option httpchk GET /index.html HTTP/1.0\\r\\n\n"
+              "    http-check expect rstatus 418\n"
+              "    fullconn {maxconn}\n"
+              "    option allbackups\n"
+              "    timeout connect 5000\n"
+              "    timeout server 50000\n"
+              "    server sample_member_id_1 10.0.0.99:82 "
+              "weight 13 check inter 30s fall 3 rise 2 "
+              "cookie sample_member_id_1\n"
+              "    server sample_member_id_2 10.0.0.98:82 "
+              "weight 13 check inter 30s fall 3 rise 2 "
+              "cookie sample_member_id_2\n\n").format(
+            maxconn=constants.HAPROXY_DEFAULT_MAXCONN)
+        rendered_obj = self.jinja_cfg.render_loadbalancer_obj(
+            sample_configs_combined.sample_amphora_tuple(),
+            [sample_configs_combined.sample_listener_tuple(
+                proto='TERMINATED_HTTPS', tls=True, tls_ciphers=None,
+                tls_versions=None)],
+            tls_certs={'cont_id_1':
+                       sample_configs_combined.sample_tls_container_tuple(
+                           id='tls_container_id',
+                           certificate='ImAalsdkfjCert',
+                           private_key='ImAsdlfksdjPrivateKey',
+                           primary_cn="FakeCN")})
+        self.assertEqual(
+            sample_configs_combined.sample_base_expected_config(
+                frontend=fe, backend=be),
+            rendered_obj)
+
+    def test_render_template_tls_alpn(self):
+        conf = oslo_fixture.Config(cfg.CONF)
+        conf.config(group="haproxy_amphora", base_cert_dir='/fake_cert_dir')
+        FAKE_CRT_LIST_FILENAME = os.path.join(
+            CONF.haproxy_amphora.base_cert_dir,
+            'sample_loadbalancer_id_1/sample_listener_id_1.pem')
+        alpn_protocols = ['chip', 'dale']
+        fe = ("frontend sample_listener_id_1\n"
+              "    maxconn {maxconn}\n"
+              "    redirect scheme https if !{{ ssl_fc }}\n"
+              "    bind 10.0.0.2:443 ssl crt-list {crt_list}   "
+              "ciphers {ciphers} no-sslv3 no-tlsv10 no-tlsv11 alpn {alpn}\n"
+              "    mode http\n"
+              "    default_backend sample_pool_id_1:sample_listener_id_1\n"
+              "    timeout client 50000\n").format(
+            maxconn=constants.HAPROXY_DEFAULT_MAXCONN,
+            crt_list=FAKE_CRT_LIST_FILENAME,
+            ciphers=constants.CIPHERS_OWASP_SUITE_B,
+            alpn=",".join(alpn_protocols))
+        be = ("backend sample_pool_id_1:sample_listener_id_1\n"
+              "    mode http\n"
+              "    balance roundrobin\n"
+              "    cookie SRV insert indirect nocache\n"
+              "    timeout check 31s\n"
+              "    option httpchk GET /index.html HTTP/1.0\\r\\n\n"
+              "    http-check expect rstatus 418\n"
+              "    fullconn {maxconn}\n"
+              "    option allbackups\n"
+              "    timeout connect 5000\n"
+              "    timeout server 50000\n"
+              "    server sample_member_id_1 10.0.0.99:82 "
+              "weight 13 check inter 30s fall 3 rise 2 "
+              "cookie sample_member_id_1\n"
+              "    server sample_member_id_2 10.0.0.98:82 "
+              "weight 13 check inter 30s fall 3 rise 2 "
+              "cookie sample_member_id_2\n\n").format(
+            maxconn=constants.HAPROXY_DEFAULT_MAXCONN)
+        rendered_obj = self.jinja_cfg.render_loadbalancer_obj(
+            sample_configs_combined.sample_amphora_tuple(),
+            [sample_configs_combined.sample_listener_tuple(
+                proto='TERMINATED_HTTPS', tls=True,
+                alpn_protocols=alpn_protocols)],
+            tls_certs={'cont_id_1':
+                       sample_configs_combined.sample_tls_container_tuple(
+                           id='tls_container_id',
+                           certificate='ImAalsdkfjCert',
+                           private_key='ImAsdlfksdjPrivateKey',
+                           primary_cn="FakeCN")})
+        self.assertEqual(
+            sample_configs_combined.sample_base_expected_config(
+                frontend=fe, backend=be),
+            rendered_obj)
+
+    def test_render_template_tls_no_alpn(self):
+        conf = oslo_fixture.Config(cfg.CONF)
+        conf.config(group="haproxy_amphora", base_cert_dir='/fake_cert_dir')
+        FAKE_CRT_LIST_FILENAME = os.path.join(
+            CONF.haproxy_amphora.base_cert_dir,
+            'sample_loadbalancer_id_1/sample_listener_id_1.pem')
+        fe = ("frontend sample_listener_id_1\n"
+              "    maxconn {maxconn}\n"
+              "    redirect scheme https if !{{ ssl_fc }}\n"
+              "    bind 10.0.0.2:443 ssl crt-list {crt_list}   "
+              "ciphers {ciphers} no-sslv3 no-tlsv10 no-tlsv11\n"
+              "    mode http\n"
+              "    default_backend sample_pool_id_1:sample_listener_id_1\n"
+              "    timeout client 50000\n").format(
+            maxconn=constants.HAPROXY_DEFAULT_MAXCONN,
+            crt_list=FAKE_CRT_LIST_FILENAME,
+            ciphers=constants.CIPHERS_OWASP_SUITE_B)
+        be = ("backend sample_pool_id_1:sample_listener_id_1\n"
+              "    mode http\n"
+              "    balance roundrobin\n"
+              "    cookie SRV insert indirect nocache\n"
+              "    timeout check 31s\n"
+              "    option httpchk GET /index.html HTTP/1.0\\r\\n\n"
+              "    http-check expect rstatus 418\n"
+              "    fullconn {maxconn}\n"
+              "    option allbackups\n"
+              "    timeout connect 5000\n"
+              "    timeout server 50000\n"
+              "    server sample_member_id_1 10.0.0.99:82 "
+              "weight 13 check inter 30s fall 3 rise 2 "
+              "cookie sample_member_id_1\n"
+              "    server sample_member_id_2 10.0.0.98:82 "
+              "weight 13 check inter 30s fall 3 rise 2 "
+              "cookie sample_member_id_2\n\n").format(
+            maxconn=constants.HAPROXY_DEFAULT_MAXCONN)
+        rendered_obj = self.jinja_cfg.render_loadbalancer_obj(
+            sample_configs_combined.sample_amphora_tuple(),
+            [sample_configs_combined.sample_listener_tuple(
+                proto='TERMINATED_HTTPS', tls=True,
+                alpn_protocols=None)],
+            tls_certs={'cont_id_1':
+                       sample_configs_combined.sample_tls_container_tuple(
+                           id='tls_container_id',
+                           certificate='ImAalsdkfjCert',
+                           private_key='ImAsdlfksdjPrivateKey',
+                           primary_cn="FakeCN")})
+        self.assertEqual(
+            sample_configs_combined.sample_base_expected_config(
+                frontend=fe, backend=be),
+            rendered_obj)
+
     def test_render_template_http(self):
         be = ("backend sample_pool_id_1:sample_listener_id_1\n"
               "    mode http\n"
@@ -867,6 +1085,47 @@ class TestHaproxyCfg(base.TestCase):
             rendered_obj)
 
     def test_render_template_pool_cert(self):
+        feature_compatibility = {constants.POOL_ALPN: True}
+        cert_file_path = os.path.join(self.jinja_cfg.base_crt_dir,
+                                      'sample_listener_id_1', 'fake path')
+        be = ("backend sample_pool_id_1:sample_listener_id_1\n"
+              "    mode http\n"
+              "    balance roundrobin\n"
+              "    cookie SRV insert indirect nocache\n"
+              "    timeout check 31s\n"
+              "    option httpchk GET /index.html HTTP/1.0\\r\\n\n"
+              "    http-check expect rstatus 418\n"
+              "    fullconn {maxconn}\n"
+              "    option allbackups\n"
+              "    timeout connect 5000\n"
+              "    timeout server 50000\n"
+              "    server sample_member_id_1 10.0.0.99:82 weight 13 "
+              "check inter 30s fall 3 rise 2 cookie sample_member_id_1 "
+              "{opts} alpn {alpn}\n"
+              "    server sample_member_id_2 10.0.0.98:82 weight 13 "
+              "check inter 30s fall 3 rise 2 cookie sample_member_id_2 "
+              "{opts} alpn {alpn}\n\n").format(
+            maxconn=constants.HAPROXY_DEFAULT_MAXCONN,
+            opts="ssl crt %s verify none sni ssl_fc_sni" % cert_file_path +
+                 " ciphers " + constants.CIPHERS_OWASP_SUITE_B +
+                 " no-sslv3 no-tlsv10 no-tlsv11",
+            alpn=",".join(constants.AMPHORA_SUPPORTED_ALPN_PROTOCOLS))
+        rendered_obj = self.jinja_cfg.render_loadbalancer_obj(
+            sample_configs_combined.sample_amphora_tuple(),
+            [sample_configs_combined.sample_listener_tuple(
+                pool_cert=True, tls_enabled=True,
+                backend_tls_ciphers=constants.CIPHERS_OWASP_SUITE_B)],
+            tls_certs={
+                'sample_pool_id_1':
+                    {'client_cert': cert_file_path,
+                     'ca_cert': None, 'crl': None}},
+            feature_compatibility=feature_compatibility)
+        self.assertEqual(
+            sample_configs_combined.sample_base_expected_config(backend=be),
+            rendered_obj)
+
+    def test_render_template_pool_cert_no_alpn(self):
+        feature_compatibility = {constants.POOL_ALPN: False}
         cert_file_path = os.path.join(self.jinja_cfg.base_crt_dir,
                                       'sample_listener_id_1', 'fake path')
         be = ("backend sample_pool_id_1:sample_listener_id_1\n"
@@ -888,7 +1147,8 @@ class TestHaproxyCfg(base.TestCase):
               "{opts}\n\n").format(
             maxconn=constants.HAPROXY_DEFAULT_MAXCONN,
             opts="ssl crt %s verify none sni ssl_fc_sni" % cert_file_path +
-                 " ciphers " + constants.CIPHERS_OWASP_SUITE_B)
+                 " ciphers " + constants.CIPHERS_OWASP_SUITE_B +
+                 " no-sslv3 no-tlsv10 no-tlsv11")
         rendered_obj = self.jinja_cfg.render_loadbalancer_obj(
             sample_configs_combined.sample_amphora_tuple(),
             [sample_configs_combined.sample_listener_tuple(
@@ -897,12 +1157,91 @@ class TestHaproxyCfg(base.TestCase):
             tls_certs={
                 'sample_pool_id_1':
                     {'client_cert': cert_file_path,
-                     'ca_cert': None, 'crl': None}})
+                     'ca_cert': None, 'crl': None}},
+            feature_compatibility=feature_compatibility)
+        self.assertEqual(
+            sample_configs_combined.sample_base_expected_config(backend=be),
+            rendered_obj)
+
+    def test_render_template_pool_cert_no_versions(self):
+        feature_compatibility = {constants.POOL_ALPN: True}
+        cert_file_path = os.path.join(self.jinja_cfg.base_crt_dir,
+                                      'sample_listener_id_1', 'fake path')
+        be = ("backend sample_pool_id_1:sample_listener_id_1\n"
+              "    mode http\n"
+              "    balance roundrobin\n"
+              "    cookie SRV insert indirect nocache\n"
+              "    timeout check 31s\n"
+              "    option httpchk GET /index.html HTTP/1.0\\r\\n\n"
+              "    http-check expect rstatus 418\n"
+              "    fullconn {maxconn}\n"
+              "    option allbackups\n"
+              "    timeout connect 5000\n"
+              "    timeout server 50000\n"
+              "    server sample_member_id_1 10.0.0.99:82 weight 13 "
+              "check inter 30s fall 3 rise 2 cookie sample_member_id_1 "
+              "{opts} alpn {alpn}\n"
+              "    server sample_member_id_2 10.0.0.98:82 weight 13 "
+              "check inter 30s fall 3 rise 2 cookie sample_member_id_2 "
+              "{opts} alpn {alpn}\n\n").format(
+            maxconn=constants.HAPROXY_DEFAULT_MAXCONN,
+            opts="ssl crt %s verify none sni ssl_fc_sni" % cert_file_path +
+                 " ciphers " + constants.CIPHERS_OWASP_SUITE_B,
+            alpn=",".join(constants.AMPHORA_SUPPORTED_ALPN_PROTOCOLS))
+        rendered_obj = self.jinja_cfg.render_loadbalancer_obj(
+            sample_configs_combined.sample_amphora_tuple(),
+            [sample_configs_combined.sample_listener_tuple(
+                pool_cert=True, tls_enabled=True,
+                backend_tls_ciphers=constants.CIPHERS_OWASP_SUITE_B,
+                backend_tls_versions=None)],
+            tls_certs={
+                'sample_pool_id_1':
+                    {'client_cert': cert_file_path,
+                     'ca_cert': None, 'crl': None}},
+            feature_compatibility=feature_compatibility)
         self.assertEqual(
             sample_configs_combined.sample_base_expected_config(backend=be),
             rendered_obj)
 
     def test_render_template_pool_cert_no_ciphers(self):
+        feature_compatibility = {constants.POOL_ALPN: True}
+        cert_file_path = os.path.join(self.jinja_cfg.base_crt_dir,
+                                      'sample_listener_id_1', 'fake path')
+        be = ("backend sample_pool_id_1:sample_listener_id_1\n"
+              "    mode http\n"
+              "    balance roundrobin\n"
+              "    cookie SRV insert indirect nocache\n"
+              "    timeout check 31s\n"
+              "    option httpchk GET /index.html HTTP/1.0\\r\\n\n"
+              "    http-check expect rstatus 418\n"
+              "    fullconn {maxconn}\n"
+              "    option allbackups\n"
+              "    timeout connect 5000\n"
+              "    timeout server 50000\n"
+              "    server sample_member_id_1 10.0.0.99:82 weight 13 "
+              "check inter 30s fall 3 rise 2 cookie sample_member_id_1 "
+              "{opts} alpn {alpn}\n"
+              "    server sample_member_id_2 10.0.0.98:82 weight 13 "
+              "check inter 30s fall 3 rise 2 cookie sample_member_id_2 "
+              "{opts} alpn {alpn}\n\n").format(
+            maxconn=constants.HAPROXY_DEFAULT_MAXCONN,
+            opts="ssl crt %s verify none sni ssl_fc_sni" % cert_file_path +
+                 " no-sslv3 no-tlsv10 no-tlsv11",
+            alpn=",".join(constants.AMPHORA_SUPPORTED_ALPN_PROTOCOLS))
+        rendered_obj = self.jinja_cfg.render_loadbalancer_obj(
+            sample_configs_combined.sample_amphora_tuple(),
+            [sample_configs_combined.sample_listener_tuple(
+                pool_cert=True, tls_enabled=True)],
+            tls_certs={
+                'sample_pool_id_1':
+                    {'client_cert': cert_file_path,
+                     'ca_cert': None, 'crl': None}},
+            feature_compatibility=feature_compatibility)
+        self.assertEqual(
+            sample_configs_combined.sample_base_expected_config(backend=be),
+            rendered_obj)
+
+    def test_render_template_pool_cert_no_ciphers_or_versions_or_alpn(self):
         cert_file_path = os.path.join(self.jinja_cfg.base_crt_dir,
                                       'sample_listener_id_1', 'fake path')
         be = ("backend sample_pool_id_1:sample_listener_id_1\n"
@@ -927,7 +1266,8 @@ class TestHaproxyCfg(base.TestCase):
         rendered_obj = self.jinja_cfg.render_loadbalancer_obj(
             sample_configs_combined.sample_amphora_tuple(),
             [sample_configs_combined.sample_listener_tuple(
-                pool_cert=True, tls_enabled=True)],
+                pool_cert=True, tls_enabled=True, backend_tls_versions=None,
+                backend_alpn_protocols=None)],
             tls_certs={
                 'sample_pool_id_1':
                     {'client_cert': cert_file_path,
@@ -936,7 +1276,34 @@ class TestHaproxyCfg(base.TestCase):
             sample_configs_combined.sample_base_expected_config(backend=be),
             rendered_obj)
 
+    def test_render_template_pool_no_alpn(self):
+        be = ("backend sample_pool_id_1:sample_listener_id_1\n"
+              "    mode http\n"
+              "    balance roundrobin\n"
+              "    cookie SRV insert indirect nocache\n"
+              "    timeout check 31s\n"
+              "    option httpchk GET /index.html HTTP/1.0\\r\\n\n"
+              "    http-check expect rstatus 418\n"
+              "    fullconn {maxconn}\n"
+              "    option allbackups\n"
+              "    timeout connect 5000\n"
+              "    timeout server 50000\n"
+              "    server sample_member_id_1 10.0.0.99:82 weight 13 "
+              "check inter 30s fall 3 rise 2 cookie sample_member_id_1\n"
+              "    server sample_member_id_2 10.0.0.98:82 weight 13 "
+              "check inter 30s fall 3 rise 2 cookie sample_member_id_2"
+              "\n\n").format(
+            maxconn=constants.HAPROXY_DEFAULT_MAXCONN)
+        rendered_obj = self.jinja_cfg.render_loadbalancer_obj(
+            sample_configs_combined.sample_amphora_tuple(),
+            [sample_configs_combined.sample_listener_tuple(
+                backend_alpn_protocols=None)])
+        self.assertEqual(
+            sample_configs_combined.sample_base_expected_config(backend=be),
+            rendered_obj)
+
     def test_render_template_with_full_pool_cert(self):
+        feature_compatibility = {constants.POOL_ALPN: True}
         pool_client_cert = '/foo/cert.pem'
         pool_ca_cert = '/foo/ca.pem'
         pool_crl = '/foo/crl.pem'
@@ -953,16 +1320,17 @@ class TestHaproxyCfg(base.TestCase):
               "    timeout server 50000\n"
               "    server sample_member_id_1 10.0.0.99:82 weight 13 "
               "check inter 30s fall 3 rise 2 cookie sample_member_id_1 "
-              "{opts}\n"
+              "{opts} alpn {alpn}\n"
               "    server sample_member_id_2 10.0.0.98:82 weight 13 "
               "check inter 30s fall 3 rise 2 cookie sample_member_id_2 "
-              "{opts}\n\n").format(
+              "{opts} alpn {alpn}\n\n").format(
             maxconn=constants.HAPROXY_DEFAULT_MAXCONN,
             opts="%s %s %s %s %s %s" % (
                 "ssl", "crt", pool_client_cert,
                 "ca-file %s" % pool_ca_cert,
                 "crl-file %s" % pool_crl,
-                "verify required sni ssl_fc_sni"))
+                "verify required sni ssl_fc_sni no-sslv3 no-tlsv10 no-tlsv11"),
+            alpn=",".join(constants.AMPHORA_SUPPORTED_ALPN_PROTOCOLS))
         rendered_obj = self.jinja_cfg.render_loadbalancer_obj(
             sample_configs_combined.sample_amphora_tuple(),
             [sample_configs_combined.sample_listener_tuple(
@@ -972,7 +1340,8 @@ class TestHaproxyCfg(base.TestCase):
                 'sample_pool_id_1':
                     {'client_cert': pool_client_cert,
                      'ca_cert': pool_ca_cert,
-                     'crl': pool_crl}})
+                     'crl': pool_crl}},
+            feature_compatibility=feature_compatibility)
         self.assertEqual(
             sample_configs_combined.sample_base_expected_config(backend=be),
             rendered_obj)
@@ -997,18 +1366,18 @@ class TestHaproxyCfg(base.TestCase):
 
     def test_transform_pool(self):
         in_pool = sample_configs_combined.sample_pool_tuple()
-        ret = self.jinja_cfg._transform_pool(in_pool, {})
+        ret = self.jinja_cfg._transform_pool(in_pool, {}, False)
         self.assertEqual(sample_configs_combined.RET_POOL_1, ret)
 
     def test_transform_pool_2(self):
         in_pool = sample_configs_combined.sample_pool_tuple(sample_pool=2)
-        ret = self.jinja_cfg._transform_pool(in_pool, {})
+        ret = self.jinja_cfg._transform_pool(in_pool, {}, False)
         self.assertEqual(sample_configs_combined.RET_POOL_2, ret)
 
     def test_transform_pool_http_reuse(self):
         in_pool = sample_configs_combined.sample_pool_tuple(sample_pool=2)
         ret = self.jinja_cfg._transform_pool(
-            in_pool, {constants.HTTP_REUSE: True})
+            in_pool, {constants.HTTP_REUSE: True}, False)
         expected_config = copy.copy(sample_configs_combined.RET_POOL_2)
         expected_config[constants.HTTP_REUSE] = True
         self.assertEqual(expected_config, ret)
@@ -1018,7 +1387,7 @@ class TestHaproxyCfg(base.TestCase):
         cert_path = os.path.join(self.jinja_cfg.base_crt_dir,
                                  'test_listener_id', 'pool_cert.pem')
         ret = self.jinja_cfg._transform_pool(
-            in_pool, {}, pool_tls_certs={'client_cert': cert_path})
+            in_pool, {}, False, pool_tls_certs={'client_cert': cert_path})
         expected_config = copy.copy(sample_configs_combined.RET_POOL_1)
         expected_config['client_cert'] = cert_path
         self.assertEqual(expected_config, ret)
@@ -1092,25 +1461,25 @@ class TestHaproxyCfg(base.TestCase):
     def test_transform_l7policy(self):
         in_l7policy = sample_configs_combined.sample_l7policy_tuple(
             'sample_l7policy_id_1')
-        ret = self.jinja_cfg._transform_l7policy(in_l7policy, {})
+        ret = self.jinja_cfg._transform_l7policy(in_l7policy, {}, False)
         self.assertEqual(sample_configs_combined.RET_L7POLICY_1, ret)
 
     def test_transform_l7policy_2_8(self):
         in_l7policy = sample_configs_combined.sample_l7policy_tuple(
             'sample_l7policy_id_2', sample_policy=2)
-        ret = self.jinja_cfg._transform_l7policy(in_l7policy, {})
+        ret = self.jinja_cfg._transform_l7policy(in_l7policy, {}, False)
         self.assertEqual(sample_configs_combined.RET_L7POLICY_2, ret)
 
         # test invalid action without redirect_http_code
         in_l7policy = sample_configs_combined.sample_l7policy_tuple(
             'sample_l7policy_id_8', sample_policy=2, redirect_http_code=None)
-        ret = self.jinja_cfg._transform_l7policy(in_l7policy, {})
+        ret = self.jinja_cfg._transform_l7policy(in_l7policy, {}, False)
         self.assertEqual(sample_configs_combined.RET_L7POLICY_8, ret)
 
     def test_transform_l7policy_disabled_rule(self):
         in_l7policy = sample_configs_combined.sample_l7policy_tuple(
             'sample_l7policy_id_6', sample_policy=6)
-        ret = self.jinja_cfg._transform_l7policy(in_l7policy, {})
+        ret = self.jinja_cfg._transform_l7policy(in_l7policy, {}, False)
         self.assertEqual(sample_configs_combined.RET_L7POLICY_6, ret)
 
     def test_escape_haproxy_config_string(self):
@@ -1144,7 +1513,7 @@ class TestHaproxyCfg(base.TestCase):
                 defaults=defaults, logging="\n"),
             rendered_obj)
 
-    def test_http_reuse(self):
+    def test_haproxy_cfg_1_8_vs_1_5(self):
         j_cfg = jinja_cfg.JinjaTemplater(
             base_amp_path='/var/lib/octavia',
             base_crt_dir='/var/lib/octavia/certs')
@@ -1152,12 +1521,18 @@ class TestHaproxyCfg(base.TestCase):
         sample_amphora = sample_configs_combined.sample_amphora_tuple()
         sample_proxy_listener = sample_configs_combined.sample_listener_tuple(
             be_proto='PROXY')
-        # With http-reuse
+        # With http-reuse and server-state-file
+        go = (
+            "    server-state-file /var/lib/octavia/sample_loadbalancer_id_1/"
+            "servers-state\n"
+            "    maxconn {maxconn}\n\n").format(
+            maxconn=constants.HAPROXY_DEFAULT_MAXCONN)
         be = ("backend {pool_id}:{listener_id}\n"
               "    mode http\n"
               "    http-reuse safe\n"
               "    balance roundrobin\n"
               "    cookie SRV insert indirect nocache\n"
+              "    load-server-state-from-file global\n"
               "    timeout check 31s\n"
               "    fullconn {maxconn}\n"
               "    option allbackups\n"
@@ -1178,10 +1553,11 @@ class TestHaproxyCfg(base.TestCase):
             tls_certs=None,
             haproxy_versions=("1", "8", "1"))
         self.assertEqual(
-            sample_configs_combined.sample_base_expected_config(backend=be),
+            sample_configs_combined.sample_base_expected_config(
+                global_opts=go, backend=be),
             rendered_obj)
 
-        # Without http-reuse
+        # Without http-reuse and server-state-file
         be = ("backend {pool_id}:{listener_id}\n"
               "    mode http\n"
               "    balance roundrobin\n"
@@ -1216,7 +1592,8 @@ class TestHaproxyCfg(base.TestCase):
         fe = ("frontend sample_listener_id_1\n"
               "    maxconn {maxconn}\n"
               "    redirect scheme https if !{{ ssl_fc }}\n"
-              "    bind 10.0.0.2:443 ciphers {ciphers}\n"
+              "    bind 10.0.0.2:443 ciphers {ciphers} "
+              "no-sslv3 no-tlsv10 no-tlsv11 alpn {alpn}\n"
               "    mode http\n"
               "        acl sample_l7rule_id_1 path -m beg /api\n"
               "    use_backend sample_pool_id_2:sample_listener_id_1"
@@ -1252,7 +1629,8 @@ class TestHaproxyCfg(base.TestCase):
               "    default_backend sample_pool_id_1:sample_listener_id_1\n"
               "    timeout client 50000\n".format(
                   maxconn=constants.HAPROXY_DEFAULT_MAXCONN,
-                  ciphers=constants.CIPHERS_OWASP_SUITE_B))
+                  ciphers=constants.CIPHERS_OWASP_SUITE_B,
+                  alpn=",".join(constants.AMPHORA_SUPPORTED_ALPN_PROTOCOLS)))
         be = ("backend sample_pool_id_1:sample_listener_id_1\n"
               "    mode http\n"
               "    balance roundrobin\n"
